@@ -24,6 +24,20 @@ open Eliom_content_core
 
 module JsTable = Eliommod_jstable
 
+let init_client_app ?(ssl = false) ~hostname ?(port = 80) ~full_path () =
+  Eliom_request_info.client_app_initialised := true;
+  Eliom_process.set_sitedata
+    {Eliom_types.site_dir = full_path;
+     site_dir_string = String.concat "/" full_path};
+  Eliom_process.set_info {Eliom_common.cpi_ssl = ssl ;
+                          cpi_hostname = hostname;
+                          cpi_server_port = port;
+                          cpi_original_full_path = full_path
+                         };
+  Eliom_process.set_request_template None;
+  Eliom_process.set_request_cookies Ocsigen_cookies.Cookies.empty
+
+
 (* == Auxiliaries *)
 
 let create_buffer () =
@@ -72,7 +86,7 @@ end = struct
 end
 
 module Client_value : sig
-  val find : closure_id:int64 -> instance_id:int -> _
+  val find : closure_id:int64 -> instance_id:int64 -> _
   val initialize : client_value_datum -> unit
 end = struct
 
@@ -82,10 +96,10 @@ end = struct
     Js.string (Int64.to_string closure_id)
 
   let instance_key instance_id =
-    Js.string (string_of_int instance_id)
+    Js.string (Int64.to_string instance_id)
 
   let find ~closure_id ~instance_id =
-    trace "Get client value %Ld/%d" closure_id instance_id;
+    trace "Get client value %Ld/%Ld" closure_id instance_id;
     let value =
       let instances =
         Js.Optdef.get
@@ -105,7 +119,7 @@ end = struct
     from_poly value
 
   let initialize {closure_id; instance_id; args} =
-    trace "Initialize client value %Ld/%d" closure_id instance_id;
+    trace "Initialize client value %Ld/%Ld" closure_id instance_id;
     let closure =
       try
         Client_closure.find ~closure_id
@@ -202,7 +216,7 @@ let do_request_data request_data =
                "Code generating the following client values is not linked on the client: %s"
                (String.concat ","
                   (List.map
-                     (fun d -> Printf.sprintf "%Ld/%d" d.closure_id d.instance_id)
+                     (fun d -> Printf.sprintf "%Ld/%Ld" d.closure_id d.instance_id)
                      data)))
          server_sections_data;
        Queue.iter
@@ -357,31 +371,33 @@ let raw_event_handler cv =
     let handler = (Eliom_lib.from_poly value : #Dom_html.event Js.t -> unit) in
     fun ev -> try handler ev; true with False -> false
   with Not_found ->
-    error "Client value %Ld/%d not found as event handler" closure_id instance_id
-
-let reify_caml_event node ce : #Dom_html.event Js.t -> bool = match ce with
-  | Xml.CE_call_service None -> (fun _ -> true)
+    error "Client value %Ld/%Ld not found as event handler" closure_id instance_id
+let closure_name_prefix = Eliom_lib_base.RawXML.closure_name_prefix
+let closure_name_prefix_len = String.length closure_name_prefix
+let reify_caml_event name node ce : string * (#Dom_html.event Js.t -> bool) = match ce with
+  | Xml.CE_call_service None -> name,(fun _ -> true)
   | Xml.CE_call_service (Some (`A, cookies_info, tmpl)) ->
-      (fun ev ->
+      name,(fun ev ->
         let node = Js.Opt.get (Dom_html.CoerceTo.a node) (fun () -> error "not an anchor element") in
         raw_a_handler node cookies_info tmpl ev)
   | Xml.CE_call_service (Some ((`Form_get | `Form_post) as kind, cookies_info, tmpl)) ->
-      (fun ev ->
+      name,(fun ev ->
         let form = Js.Opt.get (Dom_html.CoerceTo.form node) (fun () -> error "not a form element") in
         raw_form_handler form kind cookies_info tmpl ev)
   | Xml.CE_client_closure f ->
-      (fun ev -> try f ev; true with False -> false)
+      name,(fun ev -> try f ev; true with False -> false)
   | Xml.CE_registered_closure (_, cv) ->
-      raw_event_handler cv
-
-let reify_event node ev = match ev with
-  | Xml.Raw ev -> Js.Unsafe.variable ev
-  | Xml.Caml ce -> reify_caml_event node ce
+    let name =
+      let len = String.length name in
+      if len > closure_name_prefix_len && String.sub name 0 closure_name_prefix_len = closure_name_prefix
+      then String.sub name closure_name_prefix_len (len - closure_name_prefix_len)
+      else name in
+    name, raw_event_handler cv
 
 let register_event_handler, flush_load_script =
   let add, flush = create_buffer () in
   let register node (name, ev) =
-    let f = reify_caml_event node ev in
+    let name,f = reify_caml_event name node ev in
     if name = "onload" then
       add f
     else
@@ -394,6 +410,72 @@ let register_event_handler, flush_load_script =
   in
   register, flush
 
+
+let rebuild_attrib_val = function
+  | Xml.AFloat f -> Obj.magic f
+  | Xml.AInt i ->   Obj.magic i
+  | Xml.AStr s ->   Js.string s
+  | Xml.AStrL (Xml.Space, sl) -> Js.string (String.concat " " sl)
+  | Xml.AStrL (Xml.Comma, sl) -> Js.string (String.concat "," sl)
+
+
+(* html attributes and dom properties use differant names
+   **exemple**: maxlength vs maxLenght (case sensitive).
+   - Before dom react, it was enought to set html attributes only as
+   there were no update after creation.
+   - Dom React may update attributes later.
+   Html attrib changes are not taken into account if the corresponding
+   Dom property is defined.
+   **exemple**: udpating html attribute `value` has no effect if the dom property
+   `value` has be set by the user.
+
+   =WE NEED TO SET DOM PROPERTIES=
+   -Tyxml only gives us html attribute names and we can set them safely.
+   -The name for dom properties is maybe differant.
+    We set it only if we find out that the property match_the_attribute_name / is_alerady_defined (get_prop).
+*)
+
+(* TODO: fix get_prop
+   it only work when html attribute and dom property names correspond.
+   find a way to get dom property name corresponding to html attribute
+*)
+
+let get_prop node name =
+  if Js.Optdef.test (Js.Unsafe.get node name)
+  then Some name
+  else None
+
+let iter_prop node name f =
+  match get_prop node name with
+  | Some n -> f n
+  | None -> ()
+
+let rebuild_rattrib node ra = match Xml.racontent ra with
+  | Xml.RA a ->
+    let name = Xml.aname ra in
+    let v = rebuild_attrib_val a in
+    node##setAttribute (name,v);
+  | Xml.RAReact s ->
+    let name = Js.string (Xml.aname ra) in
+    let _ = React.S.map (function
+        | None ->
+          node##removeAttribute (name);
+          iter_prop node name (Js.Unsafe.delete node);
+        | Some v ->
+          let v = rebuild_attrib_val v in
+          node##setAttribute (name,v);
+          iter_prop node name (fun name -> Js.Unsafe.set node name v);
+      ) s in ()
+  | Xml.RACamlEventHandler ev -> register_event_handler node (Xml.aname ra, ev)
+  | Xml.RALazyStr s ->
+      node##setAttribute(Js.string (Xml.aname ra), Js.string s)
+  | Xml.RALazyStrL (Xml.Space, l) ->
+      node##setAttribute(Js.string (Xml.aname ra), Js.string (String.concat " " l))
+  | Xml.RALazyStrL (Xml.Comma, l) ->
+    node##setAttribute(Js.string (Xml.aname ra), Js.string (String.concat "," l))
+  | Xml.RAClient _ -> assert false
+
+
 (* == Associate data to state of the History API.
 
    We store an 'id' in the state, and store data in an association
@@ -477,7 +559,7 @@ let create_request_
           ?hostname ?port ?fragment ?keep_nl_params ?nl_params get_params
       in
       `Get uri
-  | `Post ->
+  | `Post | `Put | `Delete as http_method ->
       let path, get_params, fragment, post_params =
         Eliom_uri.make_post_uri_components__
           ?absolute ?absolute_path ?https
@@ -488,7 +570,10 @@ let create_request_
       let uri =
         Eliom_uri.make_string_uri_from_components (path, get_params, fragment)
       in
-      `Post (uri, post_params)
+      (match http_method with
+      | `Post -> `Post (uri, post_params)
+      | `Put -> `Put (uri, post_params)
+      | `Delete -> `Delete (uri, post_params))
 
 let raw_call_service
     ?absolute ?absolute_path ?https ~service ?hostname ?port ?fragment
@@ -506,6 +591,14 @@ let raw_call_service
           Eliom_request.string_result
       | `Post (uri, post_params) ->
         Eliom_request.http_post
+          ?cookies_info:(Eliom_uri.make_cookies_info (https, service))
+          uri post_params Eliom_request.string_result
+      | `Put (uri, post_params) ->
+        Eliom_request.http_put
+          ?cookies_info:(Eliom_uri.make_cookies_info (https, service))
+          uri post_params Eliom_request.string_result
+      | `Delete (uri, post_params) ->
+        Eliom_request.http_delete
           ?cookies_info:(Eliom_uri.make_cookies_info (https, service))
           uri post_params Eliom_request.string_result in
   match content with
@@ -535,7 +628,9 @@ let exit_to
      get_params post_params
    with
      | `Get uri -> Eliom_request.redirect_get uri
-     | `Post (uri, post_params) -> Eliom_request.redirect_post uri post_params)
+     | `Post (uri, post_params) -> Eliom_request.redirect_post uri post_params
+     | `Put (uri, post_params) -> Eliom_request.redirect_put uri post_params
+     | `Delete (uri, post_params) -> Eliom_request.redirect_delete uri post_params)
 
 let window_open ~window_name ?window_features
     ?absolute ?absolute_path ?https ~service ?hostname ?port ?fragment
@@ -549,6 +644,8 @@ let window_open ~window_name ?window_features
     | `Get uri ->
         Dom_html.window##open_(Js.string uri, window_name, Js.Opt.option window_features)
     | `Post (uri, post_params) -> assert false
+    | `Put (uri, post_params) -> assert false
+    | `Delete (uri, post_params) -> assert false
 
 (* == Call caml service.
 
@@ -562,11 +659,11 @@ let unwrap_caml_content content =
   in
   Lwt.return (r.Eliom_types.ecs_data, r.Eliom_types.ecs_request_data)
 
-let call_caml_service
+let call_ocaml_service
     ?absolute ?absolute_path ?https ~service ?hostname ?port ?fragment
     ?keep_nl_params ?nl_params ?keep_get_na_params
     get_params post_params =
-  trace "Call caml service";
+  trace "Call OCaml service";
   lwt _, content =
     raw_call_service
       ?absolute ?absolute_path ?https ~service ?hostname ?port ?fragment
@@ -582,7 +679,7 @@ let call_caml_service
     | `Failure msg ->
       Lwt.fail (Exception_on_server msg)
 
-(* == The function [change_url_string] changes the URL, without doing a request.
+(* == Function [change_url_string] changes the URL, without doing a request.
 
    It uses the History API if present, otherwise we write the new URL
    in the fragment part of the URL (see 'redirection_script' in
@@ -608,6 +705,34 @@ let change_url_string uri =
     then Dom_html.window##location##hash <- Js.string (url_fragment_prefix^uri)
   end
 
+
+
+(* == Function [change_url] changes the URL, without doing a request.
+   It takes a GET (co-)service as parameter and its parameters.
+ *)
+
+let change_url
+    ?absolute
+    ?absolute_path
+    ?https
+    ~service
+    ?hostname
+    ?port
+    ?fragment
+    ?keep_nl_params
+    ?nl_params params =
+  change_url_string
+    (Eliom_uri.make_string_uri
+       ?absolute
+       ?absolute_path
+       ?https
+       ~service
+       ?hostname
+       ?port
+       ?fragment
+       ?keep_nl_params
+       ?nl_params params)
+
 (* == Relink
 
    Traverse the Dom representation of the page in order to register
@@ -694,9 +819,9 @@ let relink_request_nodes root =
    closure nodes is returned for application on [relink_closure_node]
    after the client values are initialized.
 *)
-let relink_page_but_closure_nodes (root:Dom_html.element Js.t) =
+let relink_page_but_client_values (root:Dom_html.element Js.t) =
   trace "Relink page";
-  let (a_nodeList,form_nodeList,process_nodeList,closure_nodeList) =
+  let (a_nodeList,form_nodeList,process_nodeList,closure_nodeList,attrib_nodeList) =
     Eliommod_dom.select_nodes root in
   Eliommod_dom.iter_nodeList a_nodeList
     (fun node -> node##onclick <- a_handler);
@@ -704,7 +829,7 @@ let relink_page_but_closure_nodes (root:Dom_html.element Js.t) =
     (fun node -> node##onsubmit <- form_handler);
   Eliommod_dom.iter_nodeList process_nodeList
     relink_process_node;
-  closure_nodeList
+  closure_nodeList,attrib_nodeList
 
 (* == Rebuild event handlers
 
@@ -717,24 +842,38 @@ let relink_page_but_closure_nodes (root:Dom_html.element Js.t) =
    when on raises [False] (cf. [raw_event_handler]).
 *)
 
+let is_closure_attrib,get_closure_name,get_closure_id =
+  let v_prefix = Eliom_lib_base.RawXML.closure_attr_prefix in
+  let v_len = String.length v_prefix in
+  let v_prefix_js = Js.string v_prefix in
+
+  let n_prefix = Eliom_lib_base.RawXML.closure_name_prefix in
+  let n_len = String.length n_prefix in
+  let n_prefix_js = Js.string n_prefix in
+
+  (fun attr ->
+    attr##value##substring(0,v_len) = v_prefix_js &&
+    attr##name##substring(0,n_len) = n_prefix_js),
+  (fun attr -> attr##name##substring_toEnd(n_len)),
+  (fun attr -> attr##value##substring_toEnd(v_len))
+
 let relink_closure_node root onload table (node:Dom_html.element Js.t) =
   trace "Relink closure node";
   let aux attr =
     (* IE8 provides [null] in node##attributes; check this first of all *)
     if Obj.magic attr then
-      ( if attr##value##substring(0,Eliom_lib_base.RawXML.closure_attr_prefix_len) =
-          Js.string Eliom_lib_base.RawXML.closure_attr_prefix
+      ( if is_closure_attrib attr
         then
-          let cid = Js.to_bytestring (attr##value##substring_toEnd(
-            Eliom_lib_base.RawXML.closure_attr_prefix_len)) in
+          let cid = Js.to_bytestring (get_closure_id attr) in
+          let name = get_closure_name attr in
           try
-            let cv = Xml.ClosureMap.find cid table in
+            let cv = Eliom_lib.RawXML.ClosureMap.find cid table in
             let closure = raw_event_handler cv in
-            if attr##name = Js.string "onload" then
+            if name = Js.string "onload" then
               (if Eliommod_dom.ancessor root node
                (* if not inside a unique node replaced by an older one *)
                then onload := closure :: !onload)
-            else Js.Unsafe.set node (attr##name) (Dom_html.handler (fun ev -> Js.bool (closure ev)))
+            else Js.Unsafe.set node name (Dom_html.handler (fun ev -> Js.bool (closure ev)))
           with Not_found ->
             error "relink_closure_node: client value %s not found" cid )
   in
@@ -750,6 +889,51 @@ let relink_closure_nodes (root : Dom_html.element Js.t) event_handlers closure_n
     ignore
       (List.for_all (fun f -> f ev) (List.rev !onload))
 
+let is_attrib_attrib,get_attrib_id =
+  let v_prefix = Eliom_lib_base.RawXML.client_attr_prefix in
+  let v_len = String.length v_prefix in
+  let v_prefix_js = Js.string v_prefix in
+
+  let n_prefix = Eliom_lib_base.RawXML.client_name_prefix in
+  let n_len = String.length n_prefix in
+  let n_prefix_js = Js.string n_prefix in
+
+  (fun attr ->
+    attr##value##substring(0,v_len) = v_prefix_js &&
+    attr##name##substring(0,n_len) = n_prefix_js),
+  (fun attr -> attr##value##substring_toEnd(v_len))
+
+let relink_attrib root table (node:Dom_html.element Js.t) =
+  trace "Relink attribute";
+  let aux attr =
+    (* IE8 provides [null] in node##attributes; check this first of all *)
+    if Obj.magic attr then
+      ( if is_attrib_attrib attr
+        then
+          let cid = Js.to_bytestring (get_attrib_id attr) in
+          try
+            let cv = Eliom_lib.RawXML.ClosureMap.find cid table in
+            let closure_id = Client_value_server_repr.closure_id cv in
+            let instance_id = Client_value_server_repr.instance_id cv in
+            begin
+              try
+                let value = Client_value.find ~closure_id ~instance_id in
+                let rattrib = (Eliom_lib.from_poly value : Eliom_content_core.Xml.attrib) in
+                rebuild_rattrib node rattrib
+              with Not_found ->
+                error "Client value %Ld/%Ld not found as event handler" closure_id instance_id
+            end
+          with Not_found ->
+            error "relink_attrib: client value %s not found" cid)
+  in
+  Eliommod_dom.iter_nodeList (node##attributes:>Dom.attr Dom.nodeList Js.t) aux
+
+
+let relink_attribs (root : Dom_html.element Js.t) attribs attrib_nodeList =
+  trace "Relink %i attributes" (attrib_nodeList##length);
+  Eliommod_dom.iter_nodeList attrib_nodeList
+    (fun node -> relink_attrib root attribs node)
+
 (* == Extract the request data and the request tab-cookies from a page
 
    See the corresponding function on the server side:
@@ -776,6 +960,8 @@ let load_data_script page =
     Firebug.console##time(Js.string "load_data_script");
   ignore (Js.Unsafe.eval_string (Js.to_string script));
   Eliom_request_info.reset_request_data ();
+  Eliom_process.reset_request_template ();
+  Eliom_process.reset_request_cookies ();
   if !Eliom_config.debug_timings then
     Firebug.console##timeEnd(Js.string "load_data_script")
 
@@ -855,8 +1041,8 @@ let set_content ?uri ?offset ?fragment content =
        (* Bind unique node (request and global) and register event
           handler.  Relinking closure nodes must take place after
           initializing the client values *)
-       let closure_nodeList = relink_page_but_closure_nodes fake_page in
-       Eliom_request_info.set_session_info js_data.Eliom_types.ejs_sess_info;
+       let closure_nodeList,attrib_nodeList = relink_page_but_client_values fake_page in
+       Eliom_request_info.set_session_info js_data.Eliom_common.ejs_sess_info;
        (* Really change page contents *)
        if !Eliom_config.debug_timings then
          Firebug.console##time(Js.string "replace_page");
@@ -868,15 +1054,18 @@ let set_content ?uri ?offset ?fragment content =
          Firebug.console##timeEnd(Js.string "replace_page");
        (* Initialize and provide client values. May need to access to
           new DOM. Necessary for relinking closure nodes *)
-       do_request_data js_data.Eliom_types.ejs_request_data;
+       do_request_data js_data.Eliom_common.ejs_request_data;
        (* Replace closure ids in document with event handlers (from client values) *)
+       let () = relink_attribs
+           Dom_html.document##documentElement
+           js_data.Eliom_common.ejs_client_attrib_table attrib_nodeList in
        let onload_closure_nodes =
          relink_closure_nodes
            Dom_html.document##documentElement
-           js_data.Eliom_types.ejs_event_handler_table closure_nodeList
+           js_data.Eliom_common.ejs_event_handler_table closure_nodeList
        in
        (* The request node table must be empty when nodes received via
-          call_caml_service are unwrapped. *)
+          call_ocaml_service are unwrapped. *)
        reset_request_nodes ();
        run_callbacks
          (Eliommod_dom.add_formdata_hack_onclick_handler ::
@@ -957,6 +1146,14 @@ let change_page
                 Eliom_request.http_post
                   ~expecting_process_page:true ?cookies_info uri p
                   Eliom_request.xml_result
+              | `Put (uri, p) ->
+                Eliom_request.http_put
+                  ~expecting_process_page:true ?cookies_info uri p
+                  Eliom_request.xml_result
+              | `Delete (uri, p) ->
+                Eliom_request.http_delete
+                  ~expecting_process_page:true ?cookies_info uri p
+                  Eliom_request.xml_result
           in
           let uri, fragment = Url.split_fragment uri in
           set_content ~uri ?fragment content )
@@ -1133,87 +1330,194 @@ type tmp_elt = {
   tmp_node_id : Xml.node_id;
 }
 
-let rebuild_attrib node name a = match a with
-  | Xml.AFloat f -> Js.Unsafe.set node (Js.string name) (Js.Unsafe.inject f)
-  | Xml.AInt i -> Js.Unsafe.set node (Js.string name) (Js.Unsafe.inject i)
-  | Xml.AStr s ->
-    node##setAttribute(Js.string name, Js.string s)
-  | Xml.AStrL (Xml.Space, sl) ->
-    node##setAttribute(Js.string name, Js.string (String.concat " " sl))
-  | Xml.AStrL (Xml.Comma, sl) ->
-    node##setAttribute(Js.string name, Js.string (String.concat "," sl))
+let delay f = Lwt.ignore_result ( Lwt.pause () >>= (fun () -> f (); Lwt.return_unit))
 
-let rebuild_rattrib node ra = match Xml.racontent ra with
-  | Xml.RA a -> rebuild_attrib node (Xml.aname ra) a
-  | Xml.RACamlEventHandler ev -> register_event_handler node (Xml.aname ra, ev)
-  | Xml.RALazyStr s ->
-      node##setAttribute(Js.string (Xml.aname ra), Js.string s)
-  | Xml.RALazyStrL (Xml.Space, l) ->
-      node##setAttribute(Js.string (Xml.aname ra), Js.string (String.concat " " l))
-  | Xml.RALazyStrL (Xml.Comma, l) ->
-      node##setAttribute(Js.string (Xml.aname ra), Js.string (String.concat "," l))
+module ReactState : sig
+  type t
+  val get_node : t -> Dom.node Js.t
+  val change_dom : t -> Dom.node Js.t -> bool
+  val init_or_update : ?state:t -> Eliom_content_core.Xml.elt -> t
+end = struct
 
-let rec rebuild_node' elt =
+  (*
+     ISSUE
+     =====
+     There is a confict when many dom react are inside each other.
+
+     let s_lvl1 = S.map (function
+     | case1 -> ..
+     | case2 -> let s_lvl2 = ... in R.node s_lvl2) ...
+     in R.node s_lvl1
+
+     both dom react will update the same dom element (call it `dom_elt`) and
+     we have to prevent an (outdated) s_lvl2 signal
+     to replace `dom_elt` (updated last by a s_lvl1 signal)
+
+     SOLUTION
+     ========
+     - an array to track versions of updates - a dom react store its version at a specify position (computed from the depth).
+     - a child can only update `dom_elt` if versions of its parents haven't changed.
+     - every time a dom react update `dom_elt`, it increment its version.
+  *)
+
+  type t = {
+    elt : Eliom_content_core.Xml.elt;  (* top element that will store the dom *)
+    global_version : int Js.js_array Js.t; (* global versions array *)
+    version_copy : int Js.js_array Js.t; (* versions when the signal started *)
+    pos : int; (* equal the depth *)
+  }
+
+  let get_node t = match Xml.get_node t.elt with
+    | Xml.DomNode d -> d
+    | _ -> assert false
+
+  let change_dom state dom =
+    let pos = state.pos in
+    let outdated = ref false in
+    for i = 0 to pos - 1 do
+      if Js.array_get state.version_copy i != Js.array_get state.global_version i (* a parent changed *)
+      then outdated := true
+    done;
+    if not !outdated
+    then
+      begin
+        if dom != get_node state
+        then
+          begin
+            (* new version *)
+            let nv = Js.Optdef.get (Js.array_get state.global_version pos) (fun _ -> 0) + 1 in
+            Js.array_set state.global_version pos nv;
+            (* Js.array_set state.version_copy pos nv; *)
+
+            Js.Opt.case ((get_node state)##parentNode)
+              (fun () -> (* no parent -> no replace needed *) ())
+              (fun parent ->
+                 Js.Opt.iter (Dom.CoerceTo.element parent) (fun parent ->
+                     (* really update the dom *)
+                     ignore ((Dom_html.element parent)##replaceChild(dom, get_node state))));
+            Xml.set_dom_node state.elt dom;
+          end;
+        false
+      end
+    else
+      begin
+        (* a parent signal changed, this dom react is outdated, do not update the dom *)
+        true
+      end
+
+  let clone_array a = a##slice_end(0)
+
+  let init_or_update ?state elt = match state with
+    | None -> (* top dom react, create a state *)
+      let global_version = jsnew Js.array_empty () in
+      let pos = 0 in
+      ignore(Js.array_set global_version pos 0);
+      let node = (Dom_html.document##createElement (Js.string "span")  :> Dom.node Js.t) in
+      Xml.set_dom_node elt node;
+      {pos;global_version;version_copy = clone_array global_version; elt}
+    | Some p -> (* child dom react, compute a state from the previous one *)
+      let pos = p.pos + 1 in
+      ignore(Js.array_set p.global_version pos 0);
+      {p with pos;version_copy = clone_array p.global_version}
+
+end
+
+let rec rebuild_node_with_state ns ?state elt =
   match Xml.get_node elt with
   | Xml.DomNode node ->
       (* assert (Xml.get_node_id node <> NoId); *)
       node
+  | Xml.ReactNode signal ->
+      let state = ReactState.init_or_update ?state elt in
+      let clear = ref None in
+      let update_signal = React.S.map (fun elt' ->
+        let dom = rebuild_node_with_state ns ~state elt' in
+        let need_cleaning = ReactState.change_dom state dom in
+        if need_cleaning then
+          match !clear with
+            | None -> ()
+            | Some s ->
+              begin
+                delay (fun () -> React.S.stop s (* clear/stop the signal we created *));
+                clear := None
+              end
+      ) signal in
+      clear := Some update_signal;
+      ReactState.get_node state
   | Xml.TyXMLNode raw_elt ->
       match Xml.get_node_id elt with
-      | Xml.NoId -> raw_rebuild_node raw_elt
+      | Xml.NoId -> raw_rebuild_node ns raw_elt
       | Xml.RequestId _ ->
           (* Do not look in request_nodes hashtbl: such elements have
              been bind while unwrapping nodes. *)
-          let node = raw_rebuild_node raw_elt in
+          let node = raw_rebuild_node ns raw_elt in
           Xml.set_dom_node elt node;
           node
       | Xml.ProcessId id ->
         let id = (Js.string id) in
         Js.Optdef.case (find_process_node id)
           (fun () ->
-            let node = raw_rebuild_node (Xml.content elt) in
+            let node = raw_rebuild_node ns (Xml.content elt) in
             register_process_node id node;
             node)
           (fun n -> (n:> Dom.node Js.t))
 
+and rebuild_node' ns e = rebuild_node_with_state ns e
 
-and raw_rebuild_node = function
+and raw_rebuild_node ns = function
   | Xml.Empty
   | Xml.Comment _ ->
       (* FIXME *)
       (Dom_html.document##createTextNode (Js.string "") :> Dom.node Js.t)
   | Xml.EncodedPCDATA s
   | Xml.PCDATA s -> (Dom_html.document##createTextNode (Js.string s) :> Dom.node Js.t)
-  | Xml.Entity s -> assert false (* FIXME *)
+  | Xml.Entity s ->
+      let entity = Dom_html.decode_html_entities (Js.string ("&" ^ s ^ ";")) in
+      (Dom_html.document##createTextNode(entity) :> Dom.node Js.t)
   | Xml.Leaf (name,attribs) ->
     let node = Dom_html.document##createElement (Js.string name) in
     List.iter (rebuild_rattrib node) attribs;
     (node :> Dom.node Js.t)
   | Xml.Node (name,attribs,childrens) ->
-    let node = Dom_html.document##createElement (Js.string name) in
+    let ns = if name = "svg" then `SVG else ns in
+    let node =
+      match ns with
+      | `HTML5 -> Dom_html.document##createElement (Js.string name)
+      | `SVG ->
+	let svg_ns = "http://www.w3.org/2000/svg" in
+	Dom_html.document##createElementNS (Js.string svg_ns, Js.string name)
+    in
     List.iter (rebuild_rattrib node) attribs;
-    List.iter (fun c -> Dom.appendChild node (rebuild_node' c)) childrens;
+    List.iter (fun c -> Dom.appendChild node (rebuild_node' ns c)) childrens;
     (node :> Dom.node Js.t)
 
-(** The first argument describes the calling function (if any) in case
-    of an error. *)
-let rebuild_node context elt =
-  let elt' = Html5.F.toelt elt in
+let rebuild_node_ns ns context elt' =
   trace "Rebuild node %a (%s)"
     (fun () e -> Eliom_content_core.Xml.string_of_node_id (Xml.get_node_id e))
     elt' context;
   if is_before_initial_load () then
-    error_any (rebuild_node' (Html5.F.toelt elt))
+    error_any (rebuild_node' ns elt')
       "Cannot apply %s%s before the document is initially loaded"
       context
       Xml.(match get_node_id elt' with
            | NoId -> " "
            | RequestId id -> " on request node "^id
            | ProcessId id -> " on global node "^id);
-  let node = Js.Unsafe.coerce (rebuild_node' elt') in
+  let node = Js.Unsafe.coerce (rebuild_node' ns elt') in
   flush_load_script ();
   node
 
+let rebuild_node_svg context elt =
+  let elt' = Svg.F.toelt elt in
+  rebuild_node_ns `SVG context elt'
+
+
+(** The first argument describes the calling function (if any) in case
+    of an error. *)
+let rebuild_node context elt =
+  let elt' = Html5.F.toelt elt in
+  rebuild_node_ns `HTML5 context elt'
+
 (******************************************************************************)
 (*                            Register unwrappers                             *)
 
