@@ -22,6 +22,19 @@
 
 open Eliom_lib
 
+(** Call this function if you want to be able to run your client side app
+    before doing the first request, that is, when the client side app
+    is not sent by the server. This may be the case for example if you are
+    developing a mobile app. The parameters correspond to the base URL of the
+    server side of your application.
+*)
+val init_client_app :
+  ?ssl:bool ->
+  hostname:string ->
+  ?port:int ->
+  full_path:Eliom_lib.Url.path -> unit -> unit
+
+
 (** Call a server side service and change the current page.
     If the service belongs to the same application,
     the client side program is not stopped, and only
@@ -33,7 +46,7 @@ val change_page :
   service:('a, 'b,
            [< Eliom_service.service_kind ],
            [< `WithSuffix | `WithoutSuffix ], 'd, 'e,
-           [< Eliom_service.registrable ], Eliom_registration.appl_service)
+           [< Eliom_service.registrable ], [< Eliom_registration.non_ocaml_service ])
           Eliom_service.service ->
   ?hostname:string ->
   ?port:int ->
@@ -45,20 +58,20 @@ val change_page :
 (** Call a server side service that return an OCaml value.
 
     If the service raises an exception, the call to the
-    [call_caml_service] raises an exception {% <<a_api|exception
+    [call_ocaml_service] raises an exception {% <<a_api|exception
     Exception_on_server>> %} whose argument describes the server-side
     exception.
     (NB that we cannot send the original exception as-it, because
     OCaml permits the marshalling of exceptions ...)
 *)
-val call_caml_service :
+val call_ocaml_service :
   ?absolute:bool ->
   ?absolute_path:bool ->
   ?https:bool ->
   service:('a, 'b,
            [< Eliom_service.service_kind ],
            [< `WithSuffix | `WithoutSuffix ], 'd, 'e,
-           [< Eliom_service.registrable ], 'return Eliom_parameter.caml)
+           [< Eliom_service.registrable ], 'return Eliom_service.ocaml_service)
     Eliom_service.service ->
   ?hostname:string ->
   ?port:int ->
@@ -78,7 +91,7 @@ val exit_to :
   service:('a, 'b,
            [< Eliom_service.service_kind ],
            [< `WithSuffix | `WithoutSuffix ], 'd, 'e,
-           [< Eliom_service.registrable ], [< Eliom_registration.non_caml_service ])
+           [< Eliom_service.registrable ], [< Eliom_registration.non_ocaml_service ])
           Eliom_service.service ->
   ?hostname:string ->
   ?port:int ->
@@ -106,6 +119,22 @@ val window_open :
   ?nl_params:Eliom_parameter.nl_params_set ->
   ?keep_get_na_params:bool -> 'a -> Dom_html.window Js.t
 
+(** Changes the URL, without doing a request.
+    It takes a GET (co-)service as parameter and its parameters.
+ *)
+val change_url :
+  ?absolute:bool ->
+  ?absolute_path:bool ->
+  ?https:bool ->
+  service:('get, unit, [< Eliom_service.get_service_kind ],
+           [< Eliom_service.suff ], 'gn, unit,
+           [< Eliom_service.registrable ], 'return) Eliom_service.service ->
+  ?hostname:string ->
+  ?port:int ->
+  ?fragment:string ->
+  ?keep_nl_params:[ `All | `None | `Persistent ] ->
+  ?nl_params:Eliom_parameter.nl_params_set -> 'get -> unit
+
 (** (low level) Call a server side service and return the content
     of the resulting HTTP frame as a string. *)
 val call_service :
@@ -124,6 +153,8 @@ val call_service :
   ?nl_params:Eliom_parameter.nl_params_set ->
   ?keep_get_na_params:bool -> 'a -> 'b -> string Lwt.t
 
+
+
 (** Registers some code to be executed after loading the client
     application, or after changing the page the next time.
 
@@ -173,19 +204,28 @@ val relink_request_nodes : Dom_html.htmlElement Js.t -> unit
 val reset_request_nodes : unit -> unit
 val force_unwrapped_elts : unit -> unit
 val do_request_data : request_data -> unit
-val relink_page_but_closure_nodes : Dom_html.element Js.t -> Dom_html.element Dom.nodeList Js.t
+val relink_page_but_client_values : Dom_html.element Js.t -> Dom_html.element Dom.nodeList Js.t * Dom_html.element Dom.nodeList Js.t
 val leave_page : unit -> unit
 val set_initial_load : unit -> unit
 
+val relink_attribs :
+  Dom_html.element Js.t ->
+  Eliom_lib.RawXML.attrib Client_value_server_repr.t Eliom_lib.RawXML.ClosureMap.t ->
+  Dom_html.element Dom.nodeList Js.t -> unit
+
 val relink_closure_nodes :
   Dom_html.element Js.t ->
-  (Dom_html.event Js.t -> unit) Client_value_server_repr.t Eliom_content_core.Xml.ClosureMap.t ->
+  (Dom_html.event Js.t -> unit) Client_value_server_repr.t Eliom_lib.RawXML.ClosureMap.t ->
   Dom_html.element Dom.nodeList Js.t ->
   (unit -> unit)
-
 val getElementById : string -> Dom.node Js.t
-val rebuild_node' : Eliom_content_core.Xml.elt -> Dom.node Js.t
+val rebuild_node' : Eliom_content_core.content_ns -> Eliom_content_core.Xml.elt -> Dom.node Js.t
 val rebuild_node : string -> 'a Eliom_content_core.Html5.elt -> < .. > Js.t
+val rebuild_node_svg : string -> 'a Eliom_content_core.Svg.elt -> < .. > Js.t
+
+(* This is necessary when generating/hacking eliom forms, e.g. when
+   deriving forms from a runtime type represantation. *)
+val form_handler : (Dom_html.element Js.t, Dom_html.event Js.t) Dom_html.event_listener
 
 module Syntax_helpers : sig
 
