@@ -24,7 +24,9 @@
 
 open Eliom_lib
 
-module Xml = struct
+type content_ns = [ `HTML5 | `SVG ]
+
+module XmlNoWrap = struct
 
   include RawXML
 
@@ -39,6 +41,7 @@ module Xml = struct
   and node =
     | DomNode of Dom.node Js.t
     | TyXMLNode of econtent
+    | ReactNode of elt React.signal
   and elt = {
     (* See Eliom_content.Html5.To_dom for the 'unwrap' function that convert
        the server's tree representation into the client one. *)
@@ -48,14 +51,17 @@ module Xml = struct
 
   let content e =
     match Lazy.force e.elt with
+    | ReactNode _
     | DomNode _ -> assert false (* TODO *)
     | TyXMLNode elt -> elt
   let get_node e = Lazy.force e.elt
   let set_dom_node elt node = elt.elt <- Lazy.lazy_from_val (DomNode node)
   let get_node_id elt = elt.node_id
 
-  let make ?(id = NoId) elt = { elt = Lazy.lazy_from_val (TyXMLNode elt); node_id = id; }
-  let make_dom ?(id = NoId) node = { elt = Lazy.lazy_from_val (DomNode node); node_id = id; }
+  let make ?(id = NoId) elt =
+    { elt = Lazy.lazy_from_val (TyXMLNode elt); node_id = id; }
+  let make_dom ?(id = NoId) node =
+    { elt = Lazy.lazy_from_val (DomNode node); node_id = id; }
   let make_lazy ?(id = NoId) lazy_elt =
     let f () =
        let elt = Lazy.force lazy_elt in
@@ -65,6 +71,9 @@ module Xml = struct
     { node_id = id; elt = Lazy.lazy_from_fun f }
   let force_lazy { elt } = ignore (Lazy.force elt)
 
+  let make_react ?(id = NoId) signal =
+    {elt = Lazy.lazy_from_val (ReactNode signal); node_id = id; }
+
   let empty () = make Empty
 
   let comment c = make (Comment c)
@@ -76,8 +85,9 @@ module Xml = struct
   let node ?(a = []) name children = make (Node (name, a, children))
   let lazy_node ?a name children = node ?a name (Eliom_lazy.force children)
 
-  let event_handler_of_function (ev: #Dom_html.event Js.t -> unit) =
-    Caml (CE_client_closure (Obj.magic ev))
+  let event_handler_of_function eh_untyped =
+    let eh : 'a Js.t -> unit = Obj.magic eh_untyped in
+    Caml (CE_client_closure eh)
 
   let end_re = Regexp.regexp_string "]]>"
 
@@ -124,6 +134,7 @@ module Xml = struct
   let set_classes_of_elt elt =
      match Lazy.force elt.elt with
       | DomNode _ -> failwith "Eliom_content_core.set_classes_of_elt"
+      | ReactNode _ -> failwith "Eliom_content_core.set_classes_of_elt"
       | TyXMLNode econtent ->
           { elt with elt = Lazy.lazy_from_val (TyXMLNode (set_classes elt.node_id econtent)) }
 
@@ -134,34 +145,149 @@ module Xml = struct
 
 end
 
+module Xml = struct
+  include XmlNoWrap
+  type 'a wrap = 'a
+end
+
 module X = Xml
 
+
+module Xml_w = struct
+  type 'a t = 'a React.signal
+  let return x = Lwt_react.S.return x
+  let bind x = Lwt_react.S.bind x
+  let fmap x = React.S.map x
+  let fmap2 x = React.S.l2 x
+  let fmap3 x = React.S.l3 x
+  let fmap4 x = React.S.l4 x
+  let fmap5 x = React.S.l5 x
+end
+module Xml_wed =
+struct
+  type 'a wrap = 'a Xml_w.t
+  type uri = Xml.uri
+  let string_of_uri = Xml.string_of_uri
+  let uri_of_string = Xml.uri_of_string
+  type aname = Xml.aname
+  type event_handler = Xml.event_handler
+  type attrib = Xml.attrib
+
+  let float_attrib name s : attrib =
+    name, Xml.RAReact (Xml_w.fmap (fun f -> Some (Xml.AFloat f)) s)
+  let int_attrib name s =
+    name, Xml.RAReact (React.S.map (fun f -> Some (Xml.AInt f)) s)
+  let string_attrib name s =
+    name, Xml.RAReact (React.S.map (fun f -> Some (Xml.AStr f)) s)
+  let space_sep_attrib name s =
+    name, Xml.RAReact (React.S.map (fun f -> Some(Xml.AStrL (Xml.Space,f))) s)
+  let comma_sep_attrib name s =
+    name, Xml.RAReact (React.S.map (fun f -> Some (Xml.AStrL (Xml.Comma,f))) s)
+  let event_handler_attrib = Xml.event_handler_attrib
+  let uri_attrib name value =
+    name, Xml.RAReact (React.S.map
+                         (fun f -> Some (Xml.AStr (Eliom_lazy.force f))) value)
+  let uris_attrib name value =
+    name,
+    Xml.RAReact (React.S.map
+                   (fun f -> Some (Xml.AStrL (Xml.Space,Eliom_lazy.force f)))
+                   value)
+
+  type elt = Xml.elt
+  type ename = Xml.ename
+
+  let empty = Xml.empty
+  let comment = Xml.comment
+  let pcdata s = Xml.make_react (React.S.map Xml.pcdata s)
+  let encodedpcdata s = Xml.make_react (React.S.map Xml.encodedpcdata s)
+  let entity = Xml.entity
+  let leaf = Xml.leaf
+  let node ?a name l = Xml.make_react (React.S.map (fun l -> Xml.node ?a name l) l)
+  let cdata = Xml.cdata
+  let cdata_script = Xml.cdata_script
+  let cdata_style = Xml.cdata_style
+end
+
+
+
 module Svg = struct
 
-  module D = Svg_f.Make(struct
-    include Xml
+  module D = struct
 
-    let make elt = make_request_node (make elt)
-    let make_lazy elt = make_request_node (make (Lazy.force elt))
+    module Raw = Svg_f.Make(struct
+        include Xml
 
-    let empty () = make Empty
+        let make elt = make_request_node (make elt)
+        let make_lazy elt = make_request_node (make (Lazy.force elt))
 
-    let comment c = make (Comment c)
-    let pcdata d = make (PCDATA d)
-    let encodedpcdata d = make (EncodedPCDATA d)
-    let entity e = make (Entity e)
+        let empty () = make Empty
 
-    let leaf ?(a = []) name =  make (Leaf (name, a))
-    let node ?(a = []) name children = make (Node (name, a, children))
-    let lazy_node ?(a = []) name children =
-      make (Node (name, a, Eliom_lazy.force children))
+        let comment c = make (Comment c)
+        let pcdata d = make (PCDATA d)
+        let encodedpcdata d = make (EncodedPCDATA d)
+        let entity e = make (Entity e)
 
-  end)
+        let leaf ?(a = []) name =  make (Leaf (name, a))
+        let node ?(a = []) name children = make (Node (name, a, children))
+        let lazy_node ?(a = []) name children =
+          make (Node (name, a, Eliom_lazy.force children))
 
-  module F = Svg_f.Make(Xml)
+      end)
 
-  type 'a elt = 'a F.elt
-  type 'a attrib = 'a F.attrib
+    include Raw
+
+    let a_onabort ev = a_onabort (X.event_handler_of_function ev)
+    let a_onclick ev = a_onclick (X.event_handler_of_function ev)
+    let a_onmousedown ev = a_onmousedown (X.event_handler_of_function ev)
+    let a_onmouseup ev = a_onmouseup (X.event_handler_of_function ev)
+    let a_onmouseover ev = a_onmouseover (X.event_handler_of_function ev)
+    let a_onmousemove ev = a_onmousemove (X.event_handler_of_function ev)
+    let a_onmouseout ev = a_onmouseout (X.event_handler_of_function ev)
+    let a_onscroll ev = a_onscroll (X.event_handler_of_function ev)
+    let a_onload ev = a_onload (X.event_handler_of_function ev)
+    let a_onresize ev = a_onresize (X.event_handler_of_function ev)
+
+  end
+
+  module F = struct
+
+    module Raw = Svg_f.Make(Xml)
+
+    include Raw
+
+    let a_onabort ev = a_onabort (X.event_handler_of_function ev)
+    let a_onclick ev = a_onclick (X.event_handler_of_function ev)
+    let a_onmousedown ev = a_onmousedown (X.event_handler_of_function ev)
+    let a_onmouseup ev = a_onmouseup (X.event_handler_of_function ev)
+    let a_onmouseover ev = a_onmouseover (X.event_handler_of_function ev)
+    let a_onmousemove ev = a_onmousemove (X.event_handler_of_function ev)
+    let a_onmouseout ev = a_onmouseout (X.event_handler_of_function ev)
+    let a_onresize ev = a_onresize (X.event_handler_of_function ev)
+    let a_onscroll ev = a_onscroll (X.event_handler_of_function ev)
+    let a_onload ev = a_onload (X.event_handler_of_function ev)
+
+  end
+
+  module R = struct
+    module Raw = Svg_f.MakeWrapped(Xml_w)(Xml_wed)
+    include Raw
+
+    let a_onabort ev = a_onabort (X.event_handler_of_function ev)
+    let a_onclick ev = a_onclick (X.event_handler_of_function ev)
+    let a_onmousedown ev = a_onmousedown (X.event_handler_of_function ev)
+    let a_onmouseup ev = a_onmouseup (X.event_handler_of_function ev)
+    let a_onmouseover ev = a_onmouseover (X.event_handler_of_function ev)
+    let a_onmousemove ev = a_onmousemove (X.event_handler_of_function ev)
+    let a_onmouseout ev = a_onmouseout (X.event_handler_of_function ev)
+    let a_onresize ev = a_onresize (X.event_handler_of_function ev)
+    let a_onscroll ev = a_onscroll (X.event_handler_of_function ev)
+    let a_onload ev = a_onload (X.event_handler_of_function ev)
+
+  end
+
+  type +'a elt = 'a F.elt
+  type 'a wrap = 'a F.wrap
+  type +'a attrib = 'a F.attrib
   type uri = F.uri
 
   module Id = struct
@@ -171,8 +297,17 @@ module Svg = struct
       D.tot (Xml.make_process_node ~id (D.toelt elt))
     let create_global_elt elt =
       D.tot (Xml.make_process_node (D.toelt elt))
+    let string_of_id x = x
   end
 
+
+  module Of_dom = struct
+    let rebuild_xml (node: 'a Js.t) : 'a F.elt =
+      Obj.magic { Xml.elt = Lazy.lazy_from_val (Xml.DomNode (node :> Dom.node Js.t)); node_id = Xml.NoId }
+    let of_element : Dom_html.element Js.t -> 'a elt = rebuild_xml
+  end
+
+
 end
 
 module Html5 = struct
@@ -197,7 +332,7 @@ module Html5 = struct
       let lazy_node ?(a = []) name children =
         make (Node (name, a, Eliom_lazy.force children))
 
-    end)(Svg.D)
+    end)(Svg.D.Raw)
 
     include Raw
 
@@ -283,9 +418,19 @@ module Html5 = struct
 
   end
 
+
+
+  module R = struct
+
+    let node s = Xml.make_react s
+
+    module Raw = Html5_f.MakeWrapped(Xml_w)(Xml_wed)(Svg.R)
+    include Raw
+  end
+
   module F = struct
 
-    module Raw = Html5_f.Make(Xml)(Svg.F)
+    module Raw = Html5_f.Make(Xml)(Svg.F.Raw)
     include Raw
 
     let a_onabort ev = a_onabort (X.event_handler_of_function ev)
@@ -371,6 +516,7 @@ module Html5 = struct
   end
 
   type +'a elt = 'a F.elt
+  type 'a wrap = 'a F.wrap
   type +'a attrib = 'a F.attrib
   type uri = F.uri
 
@@ -477,7 +623,3 @@ module Html5 = struct
   let set_classes_of_elt elt = F.tot (X.set_classes_of_elt (F.toelt elt))
 
 end
-
-
-
-
