@@ -20,6 +20,8 @@
 (** XML building and deconstructing. Cf. {% <<a_api subproject="server" |
     module Eliom_content_core.Xml >> %}. *)
 
+type content_ns = [ `HTML5 | `SVG ]
+
 module Xml : sig
 
   type uri = string
@@ -42,6 +44,7 @@ module Xml : sig
 
   type ename = string
   type elt
+  type 'a wrap = 'a
   type econtent = private
     | Empty
     | Comment of string
@@ -57,7 +60,7 @@ module Xml : sig
     ( [ `A | `Form_get | `Form_post ]
       * (bool * string list) option
       * string option) option Eliom_lazy.request -> event_handler
-  val event_handler_of_function : (#Dom_html.event Js.t -> unit) -> event_handler
+  val event_handler_of_function : (Dom_html.event Js.t -> unit) -> event_handler
 
   type separator = Space | Comma
   type acontent = private
@@ -65,13 +68,16 @@ module Xml : sig
     | AInt of int
     | AStr of string
     | AStrL of separator * string list
+
   val acontent : attrib -> acontent
 
   type racontent =
     | RA of acontent
+    | RAReact of acontent option React.signal
     | RACamlEventHandler of Dom_html.event caml_event_handler
     | RALazyStr of string Eliom_lazy.request
     | RALazyStrL of separator * string Eliom_lazy.request list
+    | RAClient of string * attrib option * attrib Eliom_lib.Client_value_server_repr.t
   val racontent : attrib -> racontent
 
   val aname : attrib -> aname
@@ -111,6 +117,7 @@ module Xml : sig
   val make_dom : ?id:node_id -> Dom.node Js.t -> elt
   val make_lazy : ?id:node_id -> elt lazy_t -> elt
   val force_lazy : elt -> unit
+  val make_react : ?id:node_id -> elt React.signal -> elt
 
   val make_process_node : ?id:string -> elt -> elt
   val make_request_node : elt -> elt
@@ -119,13 +126,10 @@ module Xml : sig
   type node =
     | DomNode of Dom.node Js.t
     | TyXMLNode of econtent
+    | ReactNode of elt React.signal
+
   val get_node : elt -> node
   val set_dom_node : elt -> Dom.node Js.t -> unit
-
-  module ClosureMap : Map.S with type key = string
-  type event_handler_table =
-    ((Dom_html.event Js.t -> unit) Eliom_lib.Client_value_server_repr.t) ClosureMap.t
-
 end
 
 (** Building SVG tree. *)
@@ -137,7 +141,8 @@ module Svg : sig
       application. *)
 
   type +'a elt
-  type 'a attrib
+  type 'a wrap = 'a
+  type +'a attrib
   type uri = Xml.uri
 
   (** {2 Functional semantics} *)
@@ -145,25 +150,77 @@ module Svg : sig
   (** Typed interface for building valid SVG tree (functional
       semantics). See {% <<a_api project="tyxml" | module type
       Svg_sigs.T >> %}. *)
-  module F : Svg_sigs.T with type Xml.uri = Xml.uri
-                        and type Xml.event_handler = Xml.event_handler
-                        and type Xml.attrib = Xml.attrib
-                        and type Xml.elt = Xml.elt
-                        with type 'a elt = 'a elt
-                        and type 'a attrib = 'a attrib
-                        and type uri = uri
+  module F : sig
+    module Raw : Svg_sigs.T with type Xml.uri = Xml.uri
+                            and type Xml.event_handler = Xml.event_handler
+                            and type Xml.attrib = Xml.attrib
+                            and type Xml.elt = Xml.elt
+                              with type +'a elt = 'a elt
+                              and type 'a Xml.wrap = 'a
+                              and type 'a wrap = 'a
+                              and type +'a attrib = 'a attrib
+                              and type uri = uri
+
+    (** See {% <<a_api project="tyxml" | module type Svg_sigs.T >> %}. *)
+    include module type of Raw
+
+    (** {2 Event handlers} *)
+
+    (** Redefine event handler attributes to simplify their usage. *)
+    include "sigs/eliom_svg_event_handler.mli"
+
+  end
 
   (** {2 DOM semantics} *)
 
   (** Typed interface for building valid SVG tree (DOM semantics). See
       {% <<a_api project="tyxml" | module type Svg_sigs.T >> %}. *)
-  module D : Svg_sigs.T with type Xml.uri = Xml.uri
-                        and type Xml.event_handler = Xml.event_handler
-                        and type Xml.attrib = Xml.attrib
-                        and type Xml.elt = Xml.elt
-                        with type 'a elt = 'a elt
-                        and type 'a attrib = 'a attrib
-                        and type uri = uri
+  module D : sig
+
+    module Raw : Svg_sigs.T with type Xml.uri = Xml.uri
+                            and type Xml.event_handler = Xml.event_handler
+                            and type Xml.attrib = Xml.attrib
+                            and type Xml.elt = Xml.elt
+                              with type +'a elt = 'a elt
+                              and type 'a Xml.wrap = 'a
+                              and type 'a wrap = 'a
+                              and type +'a attrib = 'a attrib
+                              and type uri = uri
+
+    (** See {% <<a_api project="tyxml" | module type Svg_sigs.T >> %}. *)
+    include module type of Raw
+
+    (** {2 Event handlers} *)
+
+    (** Redefine event handler attributes to simplify their usage. *)
+    include "sigs/eliom_svg_event_handler.mli"
+
+  end
+
+  (** {2 Reactive DOM} *)
+
+  (** Typed interface for building valid reactive SVG tree. *)
+  module R : sig
+
+    module Raw : Svg_sigs.T with type Xml.uri = Xml.uri
+                            and type Xml.event_handler = Xml.event_handler
+                            and type Xml.attrib = Xml.attrib
+                            and type Xml.elt = Xml.elt
+                              with type +'a elt = 'a elt
+                              and type 'a Xml.wrap = 'a React.signal
+                              and type 'a wrap = 'a React.signal
+                              and type +'a attrib = 'a attrib
+                              and type uri = uri
+
+    (** See {% <<a_api project="tyxml" | module type Svg_sigs.T >> %}. *)
+    include module type of Raw
+
+    (** {2 Event handlers} *)
+
+    (** Redefine event handler attributes to simplify their usage. *)
+    include "sigs/eliom_svg_event_handler.mli"
+
+  end
 
   (** {2 Global node} *)
 
@@ -177,6 +234,13 @@ module Svg : sig
     val create_named_elt: id:'a id -> 'a elt -> 'a elt
     (** See {!Eliom_content.Html5.Id.create_global_elt} *)
     val create_global_elt: 'a elt -> 'a elt
+
+    (**/**)
+    val string_of_id : 'a id -> string
+  end
+
+  module Of_dom : sig
+    val of_element: Dom_html.element Js.t -> 'a elt
   end
 
 end
@@ -190,6 +254,7 @@ module Html5 : sig
       application. *)
 
   type +'a elt
+  type 'a wrap = 'a
   type +'a attrib
   type uri = Xml.uri
 
@@ -203,9 +268,11 @@ module Html5 : sig
                    and type Xml.event_handler = Xml.event_handler
                    and type Xml.attrib = Xml.attrib
                    and type Xml.elt = Xml.elt
-                   with module Svg := Svg.F
-                   with type 'a elt = 'a elt
-                   and type 'a attrib = 'a attrib
+                   with module Svg := Svg.F.Raw
+                   with type +'a elt = 'a elt
+                   and type 'a Xml.wrap = 'a
+                   and type 'a wrap = 'a
+                   and type +'a attrib = 'a attrib
                    and type uri = uri
 
     (** See {% <<a_api project="tyxml" | module type Html5_sigs.T >> %}. *)
@@ -225,6 +292,7 @@ module Html5 : sig
 
   end
 
+
   (** Typed interface for building valid HTML5 tree (DOM semantics). See
       {% <<a_api project="tyxml" | module type Html5_sigs.T >> %}. *)
   module D: sig
@@ -234,9 +302,11 @@ module Html5 : sig
                    and type Xml.event_handler = Xml.event_handler
                    and type Xml.attrib = Xml.attrib
                    and type Xml.elt = Xml.elt
-                   and module Svg := Svg.D
-                   and type 'a elt = 'a elt
-                   and type 'a attrib = 'a attrib
+                   and module Svg := Svg.D.Raw
+                   and type +'a elt = 'a elt
+                   and type 'a Xml.wrap = 'a
+                   and type 'a wrap = 'a
+                   and type +'a attrib = 'a attrib
                    and type uri = uri
     include module type of Raw (*BB TODO Hide untyped [input]. *)
 
@@ -252,6 +322,33 @@ module Html5 : sig
 
   end
 
+  (** Typed interface for building valid HTML5 tree from
+      {{: http://erratique.ch/software/react} React } signals.
+      HTML5's trees are automatically updated whenever
+      corresponding signals change.
+
+      {% <<a_api project="tyxml" | module type Html5_sigs.T >> %}. *)
+
+  module R: sig
+
+    (** the function [node s] create an HTML5 [elt] from a signal [s].
+    The resulting HTML5 [elt] can then be used like anyother HTML5 [elt] *)
+    val node : 'a elt React.signal -> 'a elt
+
+    module Raw : Html5_sigs.T
+                   with type Xml.uri = Xml.uri
+                   and type Xml.event_handler = Xml.event_handler
+                   and type Xml.attrib = Xml.attrib
+                   and type Xml.elt = Xml.elt
+                   and module Svg := Svg.D.Raw
+                   and type +'a elt = 'a elt
+                   and type 'a Xml.wrap = 'a React.signal
+                   and type 'a wrap = 'a React.signal
+                   and type +'a attrib = 'a attrib
+                   and type uri = uri
+    include module type of Raw
+  end
+
 
   (** Node identifiers *)
   module Id : sig
