@@ -50,7 +50,7 @@ exception Eliom_Internal_Error of string
 *)
 exception Exception_on_server of string
 
-type file_info
+type file_info = File.file Js.t
 
 val to_json : ?typ:'a -> 'b -> string
 val of_json : ?typ:'a -> string -> 'b
@@ -92,7 +92,7 @@ val trace : ('a, unit, string, unit) format4 -> 'a
 val lwt_ignore : ?message:string -> unit Lwt.t -> unit
 
 val encode_form_value : 'a -> string
-val unmarshal_js_var : string -> 'a
+val unmarshal_js : Js.js_string Js.t -> 'a
 
 val encode_header_value : 'a -> string
 
