@@ -180,16 +180,19 @@ val suffix_const :
     string ->
       (unit, [ `WithoutSuffix ], [ `One of unit ] param_name) params_type
 
-type 'a caml
+type 'a ocaml
 
-val caml :
+val ocaml :
   string ->
   'a Deriving_Json.t ->
-  ('a, [ `WithoutSuffix ], [ `One of 'a caml ] param_name) params_type
+  ('a, [ `WithoutSuffix ], [ `One of 'a ocaml ] param_name) params_type
+
+type raw_post_data =
+  ((string * string) * (string * string) list) option *
+    string Ocsigen_stream.t option
 
 val raw_post_data :
-  (((string * string) * (string * string) list) option *
-      string Ocsigen_stream.t option,
+  (raw_post_data,
    [ `WithoutSuffix ], no_param_name) params_type
 
 type ('a, +'tipo, +'names) non_localized_params
@@ -225,24 +228,25 @@ val add_pref_params :
         ('a, 'b, 'c) params_type
 
 val construct_params :
-  (string * string) list String.Table.t ->
+  (string * Eliommod_parameters.param) list String.Table.t ->
   ('a, [< `WithSuffix | `WithoutSuffix ], 'b) params_type ->
   'a -> string list option * string
 
-val construct_params_string : (string * string) list -> string
+val construct_params_string :
+  (string * Eliommod_parameters.param) list -> string
 
 val construct_params_list_raw :
-  (string * string) list String.Table.t ->
+  (string * Eliommod_parameters.param) list String.Table.t ->
   ('a, [< `WithSuffix | `WithoutSuffix ], 'b) params_type ->
   'a -> string list option *
-  (string * string) list String.Table.t *
-     (string * string) list
+  (string * Eliommod_parameters.param) list String.Table.t *
+     (string * Eliommod_parameters.param) list
 
 val construct_params_list :
-  (string * string) list String.Table.t ->
+  (string * Eliommod_parameters.param) list String.Table.t ->
   ('a, [< `WithSuffix | `WithoutSuffix ], 'b) params_type ->
   'a ->
-  string list option * (string * string) list
+  string list option * (string * Eliommod_parameters.param) list
 
 val make_params_names :
     ('a, 'b, 'c) params_type -> bool * 'c
@@ -255,14 +259,15 @@ val nl_prod :
   ('a * 's, 'su, 'an * 'sn) params_type
 
 val remove_from_nlp :
-  (string * string) list String.Table.t ->
+  (string * 'c) list String.Table.t ->
   ('a, [< `WithSuffix | `WithoutSuffix ], 'b) params_type ->
-  (string * string) list String.Table.t
+  (string * 'c) list String.Table.t
 
 val table_of_nl_params_set : nl_params_set ->
-  (string * string) list String.Table.t
+  (string * Eliommod_parameters.param) list String.Table.t
 
-val list_of_nl_params_set : nl_params_set -> (string * string) list
+val list_of_nl_params_set :
+  nl_params_set -> (string * Eliommod_parameters.param) list
 
 val string_of_nl_params_set : nl_params_set -> string
 
