@@ -26,7 +26,7 @@ class type ['a,'b] weakMap = object
   method has : 'a -> bool Js.t Js.meth
 end
 
-let weakMap : ('a,'b) weakMap Js.t Js.constr = Js.Unsafe.variable "window.WeakMap"
+let weakMap : ('a,'b) weakMap Js.t Js.constr = Js.Unsafe.global##_WeakMap
 
 let map : (Obj.t,Obj.t) weakMap Js.t = jsnew weakMap ()
 
@@ -175,11 +175,11 @@ let late_unwrap_value unwrap_id predicate new_value =
   else
     Js.array_set occurrences_table unwrap_id all_occurrences'
 
-external raw_unmarshal_and_unwrap
+let raw_unmarshal_and_unwrap
   : (unwrapper -> _ -> _ option) ->
     (_ -> int -> _ -> int -> unit) ->
     string -> int -> _
-        = "caml_unwrap_value_from_string"
+        = Js.Unsafe.variable "caml_unwrap_value_from_string"
 
 let unwrap s i =
   if !Eliom_config.debug_timings then
@@ -195,5 +195,5 @@ let unwrap s i =
   res
 
 
-let unwrap_js_var s =
-  unwrap (Js.to_bytestring (Js.Unsafe.variable s)) 0
+let unwrap_js s =
+  unwrap (Js.to_bytestring s) 0
