@@ -21,6 +21,8 @@ open Eliom_lib
 
 open Ocsigen_cookies
 
+exception Eliom_site_information_not_available of string
+
 (******************************************************************)
 
 type scope_hierarchy =
@@ -75,7 +77,7 @@ let scope_hierarchy_of_scope : [< user_scope ] -> scope_hierarchy =
 
 type full_state_name = (* WAS: fullsessionname *)
     (* The key in the table of states.
-       For cookies scopes, 
+       For cookies scopes,
        it is also the information in the cookie name,
        without the kind of session,
        and with the scope level (that is not in the cookie name).
@@ -101,6 +103,11 @@ type att_key_serv =
          (if the service is registered in the global table),
          the bool option is the ?secure parameter for delayed registration
          (if the service is registered in the global table) *)
+  (* The following three are for non-attached coservices
+     that have been attached on a service afterwards *)
+  | SAtt_na_named of string
+  | SAtt_na_anon of string
+  | SAtt_na_csrf_safe of (int * user_scope * bool option)
 
 type na_key_serv =
   | SNa_no (* no na information *)
@@ -169,7 +176,7 @@ let appl_name_cookie_name = "__eliom_appl_name"
 let appl_name_header_name = "X-Eliom-Application"
 let full_xhr_redir_header = "X-Eliom-Location-Full"
 let half_xhr_redir_header = "X-Eliom-Location-Half"
-let response_url_header = "X-Eliom-Location"
+let response_url_header = "X-Eliom-Url"
 
 let default_group_name = "__eliom$%@default_group"
 (*204FORMS* old implementation of forms with 204 and change_page_event
@@ -254,6 +261,17 @@ type sess_info =
 (*204FORMS*     si_internal_form: bool; *)
    }
 
+type eliom_js_page_data = {
+  ejs_global_data: global_data option;
+  ejs_request_data: request_data;
+  (* Event handlers *)
+  ejs_event_handler_table: Eliom_lib.RawXML.event_handler_table;
+  (* Client Attributes *)
+  ejs_client_attrib_table: Eliom_lib.RawXML.client_attrib_table;
+  (* Session info *)
+  ejs_sess_info: sess_info;
+}
+
 (************ unwrapping identifiers *********************)
 
 let tyxml_unwrap_id_int = Eliom_lib_base.tyxml_unwrap_id_int
