@@ -24,11 +24,9 @@ open Eliom_content_core
 open Eliom_service
 open Eliom_parameter
 
-type appl_service = [ `Appl ]
-type http_service = [ `Http ]
-type non_caml_service = [ appl_service | http_service ]
+type non_ocaml_service = Eliom_service.non_ocaml_service
 
-type input_type = 
+type input_type =
   [
   | `Url
   | `Tel
@@ -64,15 +62,16 @@ type button_type =
 (*****************************************************************************)
 
 (*BB Has nothing to do with Eliom_registration in fact, should live in something like Eliom_content_base. *)
-module Html5_forms : sig
-  module F : "sigs/eliom_html5_forms.mli"
-  module D : "sigs/eliom_html5_forms.mli"
-end = struct
+module Html5_forms(*  : sig *)
+(*   module F : "sigs/eliom_html5_forms.mli" *)
+(*   module D : "sigs/eliom_html5_forms.mli" *)
+(* end *) = struct
 
   module Html5_forms_base(Html5 : sig
-    include Html5_sigs.T with module Xml := Xml and module Svg := Svg.D
-                         and type 'a elt = 'a Html5.elt
-                         and type 'a attrib = 'a Html5.attrib
+    include Html5_sigs.T with module Xml := Xml and module Svg := Svg.D.Raw
+                         and type +'a elt = 'a Html5.elt
+                         and type 'a wrap = 'a
+                         and type +'a attrib = 'a Html5.attrib
                          and type uri = Html5.uri
     include "sigs/eliom_html5_event_handler.mli"
     type ('a, 'b, 'c) lazy_plus =
@@ -89,10 +88,10 @@ end = struct
     type form_content_elt_list = Html5_types.form_content Html5.elt list
     type form_attrib_t = Html5_types.form_attrib Html5.attrib list
 
-    type 'a a_elt = 'a Html5_types.a Html5.elt
-    type 'a a_elt_list = 'a Html5_types.a Html5.elt list
-    type 'a a_content_elt = 'a Html5.elt
-    type 'a a_content_elt_list = 'a Html5.elt list
+    type +'a a_elt = 'a Html5_types.a Html5.elt
+    type +'a a_elt_list = 'a Html5_types.a Html5.elt list
+    type +'a a_content_elt = 'a Html5.elt
+    type +'a a_content_elt_list = 'a Html5.elt list
     type a_attrib_t = Html5_types.a_attrib Html5.attrib list
 
     type link_elt = Html5_types.link Html5.elt
@@ -106,11 +105,16 @@ end = struct
 
     type input_elt = Html5_types.input Html5.elt
     type input_attrib_t = Html5_types.input_attrib Html5.attrib list
+    let a_input_required required = [Html5.a_required required]
+    let input_attrib_append = List.append
 
     type select_elt = Html5_types.select Html5.elt
     type select_content_elt = Html5_types.select_content Html5.elt
     type select_content_elt_list = Html5_types.select_content Html5.elt list
     type select_attrib_t = Html5_types.select_attrib Html5.attrib list
+    let select_content_cons hd tl = hd :: tl
+    let a_select_required required = [Html5.a_required required]
+    let select_attrib_append = List.append
 
     type button_elt = Html5_types.button Html5.elt
     type button_content_elt = Html5_types.button_content Html5.elt
@@ -272,7 +276,7 @@ end = struct
                 ?a:a_attrib attrib list ->
                 service:('get, unit, [< get_service_kind ],
                          [< suff ], 'gn, 'pn,
-                         [< registrable ], 'return) service ->
+                         [< registrable ], [< non_ocaml_service ]) service ->
                 ?hostname:string ->
                 ?port:int ->
                 ?fragment:string ->
@@ -287,7 +291,7 @@ end = struct
                 ?a:a_attrib attrib list ->
                 service:('get, unit, [< get_service_kind ],
                          [< suff ], 'gn, 'pn,
-                         [< registrable ], 'return) service ->
+                         [< registrable ], [< non_ocaml_service ]) service ->
                 ?hostname:string ->
                 ?port:int ->
                 ?fragment:string ->
@@ -330,7 +334,7 @@ end = struct
                        ?a:form_attrib attrib list ->
                        service:('get, unit, [< get_service_kind ],
                                 [<suff ], 'gn, 'pn,
-                                [< registrable ], 'return) service ->
+                                [< registrable ], [< non_ocaml_service ]) service ->
                        ?hostname:string ->
                        ?port:int ->
                        ?fragment:string ->
@@ -344,7 +348,7 @@ end = struct
                        ?a:form_attrib attrib list ->
                        service:('get, unit, [< get_service_kind ],
                                 [<suff ], 'gn, 'pn,
-                                [< registrable ], 'return) service ->
+                                [< registrable ], [< non_ocaml_service ]) service ->
                        ?hostname:string ->
                        ?port:int ->
                        ?fragment:string ->
@@ -361,7 +365,7 @@ end = struct
                            ?a:form_attrib attrib list ->
                            service:('get, unit, [< get_service_kind ],
                                     [<suff ], 'gn, 'pn,
-                                    [< registrable ], 'return) service ->
+                                    [< registrable ], [< non_ocaml_service ]) service ->
                            ?hostname:string ->
                            ?port:int ->
                            ?fragment:string ->
@@ -375,7 +379,7 @@ end = struct
                            ?a:form_attrib attrib list ->
                            service:('get, unit, [< get_service_kind ],
                                     [<suff ], 'gn, 'pn,
-                                    [< registrable ], 'return) service ->
+                                    [< registrable ], [< non_ocaml_service ]) service ->
                            ?hostname:string ->
                            ?port:int ->
                            ?fragment:string ->
@@ -393,7 +397,7 @@ end = struct
                         ?a:form_attrib attrib list ->
                         service:('get, 'post, [< post_service_kind ],
                                  [< suff ], 'gn, 'pn,
-                                 [< registrable ], 'return) service ->
+                                 [< registrable ], [< non_ocaml_service ]) service ->
                         ?hostname:string ->
                         ?port:int ->
                         ?fragment:string ->
@@ -408,7 +412,7 @@ end = struct
                         ?a:form_attrib attrib list ->
                         service:('get, 'post, [< post_service_kind ],
                                  [< suff ], 'gn, 'pn,
-                                 [< registrable ], 'return) service ->
+                                 [< registrable ], [< non_ocaml_service ]) service ->
                         ?hostname:string ->
                         ?port:int ->
                         ?fragment:string ->
@@ -425,7 +429,7 @@ end = struct
                             ?a:form_attrib attrib list ->
                             service:('get, 'post, [< post_service_kind ],
                                      [< suff ], 'gn, 'pn,
-                                     [< registrable ], 'return) service ->
+                                     [< registrable ], [< non_ocaml_service ]) service ->
                             ?hostname:string ->
                             ?port:int ->
                             ?fragment:string ->
@@ -441,7 +445,7 @@ end = struct
                             ?a:form_attrib attrib list ->
                             service:('get, 'post, [< post_service_kind ],
                                      [< suff ], 'gn, 'pn,
-                                     [< registrable ], 'return) service ->
+                                     [< registrable ], [< non_ocaml_service ]) service ->
                             ?hostname:string ->
                             ?port:int ->
                             ?fragment:string ->
@@ -622,6 +626,12 @@ end = struct
                            ?a:(input_attrib attrib list ) -> ?checked:bool ->
                            name:'a -> value:string -> unit -> [> input ] elt)
 
+      let string_radio_required = (string_radio_required :
+                            ?a:(input_attrib attrib list ) -> ?checked:bool ->
+                           name:'a -> value:string -> unit -> input elt :>
+                           ?a:(input_attrib attrib list ) -> ?checked:bool ->
+                           name:'a -> value:string -> unit -> [> input ] elt)
+
       let int_radio = (int_radio :
                          ?a:(input_attrib attrib list ) -> ?checked:bool ->
                         name:'a -> value:int -> unit -> input elt :>
@@ -679,60 +689,72 @@ end = struct
 
       let raw_select = (raw_select :
                           ?a:select_attrib attrib list ->
+                         ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                          name:string ->
                          string select_opt ->
                          string select_opt list -> select elt :>
                          ?a:select_attrib attrib list ->
+                         ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                          name:string ->
                          string select_opt ->
                          string select_opt list -> [> select ] elt)
 
       let int_select = (int_select :
                           ?a:select_attrib attrib list ->
+                         ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                          name:'a ->
                          int select_opt ->
                          int select_opt list -> select elt :>
                          ?a:select_attrib attrib list ->
+                         ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                          name:'a ->
                          int select_opt ->
                          int select_opt list -> [> select ] elt)
 
       let int32_select = (int32_select :
                             ?a:select_attrib attrib list ->
+                           ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                            name:'a ->
                            int32 select_opt ->
                            int32 select_opt list -> select elt :>
                            ?a:select_attrib attrib list ->
+                          ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                            name:'a ->
                            int32 select_opt ->
                            int32 select_opt list -> [> select ] elt)
 
       let int64_select = (int64_select :
                             ?a:select_attrib attrib list ->
+                           ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                            name:'a ->
                            int64 select_opt ->
                            int64 select_opt list -> select elt :>
                            ?a:select_attrib attrib list ->
+                           ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                            name:'a ->
                            int64 select_opt ->
                            int64 select_opt list -> [> select ] elt)
 
       let float_select = (float_select :
                             ?a:select_attrib attrib list ->
+                           ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                            name:'a ->
                            float select_opt ->
                            float select_opt list -> select elt :>
                            ?a:select_attrib attrib list ->
+                           ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                            name:'a ->
                            float select_opt ->
                            float select_opt list -> [> select ] elt)
 
       let string_select = (string_select :
                              ?a:select_attrib attrib list ->
+                            ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                             name:'a ->
                             string select_opt ->
                             string select_opt list -> select elt :>
                             ?a:select_attrib attrib list ->
+                            ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                             name:'a ->
                             string select_opt ->
                             string select_opt list -> [> select ] elt)
@@ -740,11 +762,13 @@ end = struct
       let user_type_select = (user_type_select :
                                 ('a -> string) ->
                                ?a:select_attrib attrib list ->
+                               ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                name:'b ->
                                'a select_opt ->
                                'a select_opt list -> select elt :>
                                ('a -> string) ->
                                ?a:select_attrib attrib list ->
+                               ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                name:'b ->
                                'a select_opt ->
                                'a select_opt list -> [> select ] elt)
@@ -752,60 +776,72 @@ end = struct
 
       let raw_multiple_select = (raw_multiple_select :
                                    ?a:select_attrib attrib list ->
+                                  ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                   name:string ->
                                   string select_opt ->
                                   string select_opt list -> select elt :>
                                   ?a:select_attrib attrib list ->
+                                  ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                   name:string ->
                                   string select_opt ->
                                   string select_opt list -> [> select ] elt)
 
       let int_multiple_select = (int_multiple_select :
                                    ?a:select_attrib attrib list ->
+                                  ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                   name:'a ->
                                   int select_opt ->
                                   int select_opt list -> select elt :>
                                   ?a:select_attrib attrib list ->
+                                  ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                   name:'a ->
                                   int select_opt ->
                                   int select_opt list -> [> select ] elt)
 
       let int32_multiple_select = (int32_multiple_select :
                                      ?a:select_attrib attrib list ->
+                                    ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                     name:'a ->
                                     int32 select_opt ->
                                     int32 select_opt list -> select elt :>
                                     ?a:select_attrib attrib list ->
+                                    ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                     name:'a ->
                                     int32 select_opt ->
                                     int32 select_opt list -> [> select ] elt)
 
       let int64_multiple_select = (int64_multiple_select :
                                      ?a:select_attrib attrib list ->
+                                    ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                     name:'a ->
                                     int64 select_opt ->
                                     int64 select_opt list -> select elt :>
                                     ?a:select_attrib attrib list ->
+                                    ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                     name:'a ->
                                     int64 select_opt ->
                                     int64 select_opt list -> [> select ] elt)
 
       let float_multiple_select = (float_multiple_select :
                                      ?a:select_attrib attrib list ->
+                                    ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                     name:'a ->
                                     float select_opt ->
                                     float select_opt list -> select elt :>
                                     ?a:select_attrib attrib list ->
+                                    ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                     name:'a ->
                                     float select_opt ->
                                     float select_opt list -> [> select ] elt)
 
       let string_multiple_select = (string_multiple_select :
                                       ?a:select_attrib attrib list ->
+                                     ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                      name:'a ->
                                      string select_opt ->
                                      string select_opt list -> select elt :>
                                      ?a:select_attrib attrib list ->
+                                     ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                      name:'a ->
                                      string select_opt ->
                                      string select_opt list -> [> select ] elt)
@@ -813,11 +849,13 @@ end = struct
       let user_type_multiple_select = (user_type_multiple_select :
                                          ('a -> string) ->
                                         ?a:select_attrib attrib list ->
+                                        ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                         name:'b ->
                                         'a select_opt ->
                                         'a select_opt list -> select elt :>
                                         ('a -> string) ->
                                         ?a:select_attrib attrib list ->
+                                        ?required:Html5_types.pcdata Eliom_content_core.Html5.elt ->
                                         name:'b ->
                                         'a select_opt ->
                                         'a select_opt list -> [> select ] elt)
@@ -900,7 +938,7 @@ end = struct
 
   end)
 
-  module MakeApplForms(Forms: "sigs/eliom_html5_forms.mli") = struct
+  module MakeApplForms(Forms: "sigs/eliom_html5_forms_.mli") = struct
 
     include Forms
 
@@ -1001,4 +1039,3 @@ end = struct
   module F = MakeApplForms(Open_Html5_forms(Eliom_mkforms.MakeForms(Html5_forms_base(Html5.F))))
   module D = MakeApplForms(Open_Html5_forms(Eliom_mkforms.MakeForms(Html5_forms_base(Html5.D))))
 end
-
