@@ -15,9 +15,9 @@ type form_content_elt
 type form_content_elt_list
 type form_attrib_t
 
-type 'a a_elt
-type 'a a_content_elt
-type 'a a_content_elt_list
+type +'a a_elt
+type +'a a_content_elt
+type +'a a_content_elt_list
 type a_attrib_t
 
 type link_elt
@@ -106,7 +106,7 @@ type for_attrib
 
     The optional parameter [keep_nl_params] allows one to override the
     [keep_nl_params] parameter used when creating the [service], see
-    {!val:Eliom_service.service} for a detailled description.
+    {!Eliom_service.Http.service} for a detailled description.
 
     The optional parameter [nl_params] allows one to add non localized
     GET parameter to the URL.  See the eliom manual for more
@@ -198,7 +198,7 @@ val make_uri_components :
   ?keep_nl_params:[ `All | `Persistent | `None ] ->
   ?nl_params: Eliom_parameter.nl_params_set ->
   'get ->
-  string * (string * string) list * string option
+  string * (string * Eliommod_parameters.param) list * string option
 
 (** Same a {!make_uri_components}, but also returns a list of post
     parameters. *)
@@ -209,6 +209,12 @@ val make_post_uri_components :
   service:('get, 'post, [< post_service_kind ],
            [< suff ], 'gn, 'pn,
            [< registrable ], 'return) service ->
+(*  service:('get, 'post,
+           [< `Attached of ([> `External ], 'c) Eliom_service.a_s
+           | `Nonattached of [< `Post ] Eliom_service.na_s ],
+           [< Eliom_service.suff ], 'gn, 'pn,
+           [< Eliom_service.registrable ], 'return)
+    Eliom_service.service -> *)
   ?hostname:string ->
   ?port:int ->
   ?fragment:string ->
@@ -217,7 +223,8 @@ val make_post_uri_components :
   ?keep_get_na_params:bool ->
   'get ->
   'post ->
-  string * (string * string) list * string option * (string * string) list
+  string * (string * Eliommod_parameters.param) list * string option *
+    (string * Eliommod_parameters.param) list
 
 (**/**)
 (** Creates the string corresponding to the beginning of the URL,
@@ -259,7 +266,7 @@ val a :
   ?a:a_attrib_t ->
   service:('get, unit, [< get_service_kind ],
            [< suff ], 'd, unit,
-           [< registrable ], 'f)
+           [< registrable ], [< non_ocaml_service])
     service ->
   ?hostname:string ->
   ?port:int ->
@@ -277,7 +284,7 @@ val a :
     If the CSS is generated by an Eliom service, use {!make_uri} to
     calculate the service URI. If the CSS is a static file, you may
     also use {!Eliom_service.static_dir} or
-    {!Eliom_service.external_service} to abstract the file with a
+    {!Eliom_service.Http.external_service} to abstract the file with a
     service.
 
     The optional parameter [~a] allows one to add extra HTML attributes to
@@ -291,7 +298,7 @@ val css_link : ?a:link_attrib_t -> uri:uri -> unit -> link_elt
     If the script content is generated by an Eliom service, use
     {!make_uri} to calculate the service URI. If it is a static file,
     you may also use {!Eliom_service.static_dir} or
-    {!Eliom_service.external_service} to abstract the file with a
+    {!Eliom_service.Http.external_service} to abstract the file with a
     service.
 
     The optional parameter [~a] allows one to add extra HTML attributes to
@@ -330,7 +337,7 @@ val get_form :
   ?a:form_attrib_t ->
   service:('get, unit, [< get_service_kind ],
            [<suff ], 'gn, 'pn,
-           [< registrable ], 'return) service ->
+           [< registrable ], [< non_ocaml_service]) service ->
   ?hostname:string ->
   ?port:int ->
   ?fragment:string ->
@@ -349,7 +356,7 @@ val lwt_get_form :
   ?a:form_attrib_t ->
   service:('get, unit, [< get_service_kind ],
            [<suff ], 'gn, 'pn,
-           [< registrable ], 'return) service ->
+           [< registrable ], [< non_ocaml_service]) service ->
   ?hostname:string ->
   ?port:int ->
   ?fragment:string ->
@@ -370,7 +377,7 @@ val lwt_get_form :
     The optional parameter [~a] allows one to add HTML attributes to the
     generated node.
 
-    See {!Eliom_service.post_coservice'} for a description of the
+    See {!Eliom_service.Http.post_coservice'} for a description of the
     [~keep_get_na_params] optional parameter ; see {!get_form} for
     [~xhr] and see {!make_uri} for other optional parameters.
 *)
@@ -381,7 +388,7 @@ val post_form :
   ?a:form_attrib_t ->
   service:('get, 'post, [< post_service_kind ],
            [< suff ], 'gn, 'pn,
-           [< registrable ], 'return) service ->
+           [< registrable ], [< non_ocaml_service]) service ->
   ?hostname:string ->
   ?port:int ->
   ?fragment:string ->
@@ -402,7 +409,7 @@ val lwt_post_form :
   ?a:form_attrib_t ->
   service:('get, 'post, [< post_service_kind ],
            [< suff ], 'gn, 'pn,
-           [< registrable ], 'return) service ->
+           [< registrable ], [< non_ocaml_service]) service ->
   ?hostname:string ->
   ?port:int ->
   ?fragment:string ->
@@ -594,6 +601,11 @@ val string_radio :
   name:[ `Radio of string ] param_name ->
   value:string -> unit -> input_elt
 
+val string_radio_required :
+  ?a:input_attrib_t -> ?checked:bool ->
+  name:[ `One of string ] param_name ->
+  value:string -> unit -> input_elt
+
 (** Creates a radio [<input>] tag with int content *)
 val int_radio :
   ?a:input_attrib_t -> ?checked:bool ->
@@ -723,6 +735,7 @@ type 'a select_opt =
 (** Creates a [<select>] tag for int values. *)
 val int_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `One of int ] param_name ->
   int select_opt ->
   int select_opt list ->
@@ -732,6 +745,7 @@ val int_select :
 (** Creates a [<select>] tag for int32 values. *)
 val int32_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `One of int32 ] param_name ->
   int32 select_opt ->
   int32 select_opt list ->
@@ -741,6 +755,7 @@ val int32_select :
 (** Creates a [<select>] tag for int64 values. *)
 val int64_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `One of int64 ] param_name ->
   int64 select_opt ->
   int64 select_opt list ->
@@ -750,6 +765,7 @@ val int64_select :
 (** Creates a [<select>] tag for float values. *)
 val float_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `One of float ] param_name ->
   float select_opt ->
   float select_opt list ->
@@ -759,6 +775,7 @@ val float_select :
 (** Creates a [<select>] tag for string values. *)
 val string_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `One of string ] param_name ->
   string select_opt ->
   string select_opt list ->
@@ -768,6 +785,7 @@ val string_select :
 (** Creates a [<select>] tag for user type values. *)
 val user_type_select : ('a -> string) ->
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `One of 'a ] param_name ->
   'a select_opt ->
   'a select_opt list ->
@@ -777,6 +795,7 @@ val user_type_select : ('a -> string) ->
 (** Creates a [<select>] tag for any (untyped) value. *)
 val raw_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:string ->
   string select_opt ->
   string select_opt list ->
@@ -786,6 +805,7 @@ val raw_select :
 (** Creates a [<select>] tag for int values. *)
 val int_multiple_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `Set of int ] param_name ->
   int select_opt ->
   int select_opt list ->
@@ -795,6 +815,7 @@ val int_multiple_select :
 (** Creates a [<select>] tag for int32 values. *)
 val int32_multiple_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `Set of int32 ] param_name ->
   int32 select_opt ->
   int32 select_opt list ->
@@ -804,6 +825,7 @@ val int32_multiple_select :
 (** Creates a [<select>] tag for int64 values. *)
 val int64_multiple_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `Set of int64 ] param_name ->
   int64 select_opt ->
   int64 select_opt list ->
@@ -813,6 +835,7 @@ val int64_multiple_select :
 (** Creates a [<select>] tag for float values. *)
 val float_multiple_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `Set of float ] param_name ->
   float select_opt ->
   float select_opt list ->
@@ -822,6 +845,7 @@ val float_multiple_select :
 (** Creates a [<select>] tag for string values. *)
 val string_multiple_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `Set of string ] param_name ->
   string select_opt ->
   string select_opt list ->
@@ -832,6 +856,7 @@ val string_multiple_select :
 val user_type_multiple_select :
   ('a -> string) ->
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:[< `Set of 'a ] param_name ->
   'a select_opt ->
   'a select_opt list ->
@@ -841,6 +866,7 @@ val user_type_multiple_select :
 (** Creates a [<select>] tag for any (untyped) value. *)
 val raw_multiple_select :
   ?a:select_attrib_t ->
+  ?required:pcdata_elt ->
   name:string ->
   string select_opt ->
   string select_opt list ->
