@@ -26,7 +26,7 @@ open Eliom_lib
 
 (*****************************************************************************)
 
-module Xml = struct
+module XmlNoWrap = struct
 
   include RawXML
 
@@ -64,7 +64,7 @@ module Xml = struct
         if Node_id_set.mem elt.node_id !node_ids_in_content then
           { elt with recontent = RE Empty }
         else elt)
-  let wrap page value = 
+  let wrap page value =
     let node_ids = ref [] in
     let rec collect_node_ids ({ elt = { node_id }} as elt) =
       if node_id <> NoId then
@@ -94,7 +94,7 @@ module Xml = struct
       wrapper_mark }
 
   let make_lazy elt =
-    { elt = 
+    { elt =
         { recontent = RELazy elt;
           node_id = NoId;
           unwrapper_mark = Eliom_wrap.create_unwrapper tyxml_unwrap_id };
@@ -161,24 +161,34 @@ module Xml = struct
 
   (** Ref tree *)
 
-  let cons_attrib att acc = match racontent att with
-    | RACamlEventHandler (CE_registered_closure (closure_id, cv)) ->
-      ClosureMap.add closure_id cv acc
-    | _ -> acc
+  let rec fold_attrib f acc elt =
+    match content elt with
+    | Empty | EncodedPCDATA _ | PCDATA _
+    | Entity _ | Comment _  -> acc
+    | Leaf (_, attribs) -> f acc attribs
+    | Node (_, attribs, elts) ->
+      let acc = f acc attribs in
+      List.fold_left (fold_attrib f) acc elts
 
   let make_event_handler_table elt =
-    let rec aux closure_acc elt =
-      let make attribs =
-        List.fold_right cons_attrib attribs closure_acc
-      in
-      match content elt with
-        | Empty | EncodedPCDATA _ | PCDATA _
-        | Entity _ | Comment _  -> closure_acc
-        | Leaf (_, attribs) -> make attribs
-        | Node (_, attribs, elts) ->
-          List.fold_left aux (make attribs) elts
+    let f acc attribs =
+      List.fold_right (fun att acc ->
+          match racontent att with
+          | RACamlEventHandler (CE_registered_closure (closure_id, cv)) ->
+            ClosureMap.add closure_id cv acc
+          | _ -> acc) attribs acc
     in
-    aux ClosureMap.empty elt
+    fold_attrib f ClosureMap.empty elt
+
+  let make_client_attrib_table elt : client_attrib_table =
+    let f acc attribs =
+      List.fold_right (fun att acc ->
+          match racontent att with
+          | RAClient (id,_,cv) ->
+            ClosureMap.add id cv acc
+          | _ -> acc) attribs acc
+    in
+    fold_attrib f ClosureMap.empty elt
 
   let set_classes node_id = function
     | Empty
@@ -200,33 +210,121 @@ module Xml = struct
 
 end
 
+module Xml = struct
+  include XmlNoWrap
+  type 'a wrap = 'a
+end
+
 module Eliom_xml = Xml
 
 module Svg = struct
 
-  module D = Svg_f.Make(struct
-    include Xml
+  module D = struct
+    module Xml' = struct
 
-    let make elt = make_request_node (make elt)
-    let make_lazy elt = make_request_node (make_lazy elt)
+      include Xml
 
-    let empty () = make Empty
+      let make elt = make_request_node (make elt)
+      let make_lazy elt = make_request_node (make_lazy elt)
 
-    let comment c = make (Comment c)
-    let pcdata d = make (PCDATA d)
-    let encodedpcdata d = make (EncodedPCDATA d)
-    let entity e = make (Entity e)
+      let empty () = make Empty
 
-    let leaf ?(a = []) name =  make (Leaf (name, a))
-    let node ?(a = []) name children = make (Node (name, a, children))
-    let lazy_node ?(a = []) name children =
-      make_lazy (Eliom_lazy.from_fun (fun () -> (Node (name, a, Eliom_lazy.force children))))
+      let comment c = make (Comment c)
+      let pcdata d = make (PCDATA d)
+      let encodedpcdata d = make (EncodedPCDATA d)
+      let entity e = make (Entity e)
 
-  end)
+      let leaf ?(a = []) name =  make (Leaf (name, a))
+      let node ?(a = []) name children = make (Node (name, a, children))
+      let lazy_node ?(a = []) name children =
+        make_lazy (Eliom_lazy.from_fun (fun () -> (Node (name, a, Eliom_lazy.force children))))
 
-  module F = Svg_f.Make(Xml)
+    end
+    module Raw = Svg_f.Make(Xml')
+
+    let client_attrib ?init (x : 'a Raw.attrib Eliom_lib.client_value) =
+      let crypto = make_cryptographic_safe_string () in
+      let empty_name = "" in
+      empty_name,Xml.RAClient (crypto,init,Obj.magic (Eliom_lib.client_value_server_repr x))
+
+    include Raw
+
+
+    let a_onabort ev = Raw.a_onabort (Eliom_xml.event_handler ev)
+    let a_onclick
+        (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+      (* Typed by the syntax extension. *)
+      Raw.a_onclick (Eliom_xml.event_handler (Obj.magic ev))
+    let a_onmousedown
+        (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+      (* Typed by the syntax extension. *)
+      Raw.a_onmousedown (Eliom_xml.event_handler (Obj.magic ev))
+    let a_onmouseup
+        (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+      (* Typed by the syntax extension. *)
+      Raw.a_onmouseup (Eliom_xml.event_handler (Obj.magic ev))
+    let a_onmouseover
+        (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+      (* Typed by the syntax extension. *)
+      Raw.a_onmouseover (Eliom_xml.event_handler (Obj.magic ev))
+    let a_onmousemove
+        (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+      (* Typed by the syntax extension. *)
+      Raw.a_onmousemove (Eliom_xml.event_handler (Obj.magic ev))
+
+    let a_onmouseout
+        (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+      (* Typed by the syntax extension. *)
+      Raw.a_onmouseout (Eliom_xml.event_handler (Obj.magic ev))
+
+    let a_onscroll ev = a_onscroll (Eliom_xml.event_handler ev)
+    let a_onload ev = a_onload (Eliom_xml.event_handler ev)
+    let a_onresize ev = a_onresize (Eliom_xml.event_handler ev)
+
+   end
+
+  module F = struct
+
+    module Raw = Svg_f.Make(Xml)
+
+      include Raw
+
+
+      let a_onabort ev = Raw.a_onabort (Eliom_xml.event_handler ev)
+      let a_onclick
+          (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+        (* Typed by the syntax extension. *)
+	Raw.a_onclick (Eliom_xml.event_handler (Obj.magic ev))
+      let a_onmousedown
+          (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+        (* Typed by the syntax extension. *)
+	Raw.a_onmousedown (Eliom_xml.event_handler (Obj.magic ev))
+      let a_onmouseup
+          (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+        (* Typed by the syntax extension. *)
+	Raw.a_onmouseup (Eliom_xml.event_handler (Obj.magic ev))
+      let a_onmouseover
+          (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+        (* Typed by the syntax extension. *)
+	Raw.a_onmouseover (Eliom_xml.event_handler (Obj.magic ev))
+      let a_onmousemove
+          (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+        (* Typed by the syntax extension. *)
+	Raw.a_onmousemove (Eliom_xml.event_handler (Obj.magic ev))
+
+      let a_onmouseout
+          (ev : (Dom_html.mouseEvent Js.t -> unit) Eliom_lib.client_value) =
+        (* Typed by the syntax extension. *)
+	Raw.a_onmouseout (Eliom_xml.event_handler (Obj.magic ev))
+
+      let a_onscroll ev = a_onscroll (Eliom_xml.event_handler ev)
+      let a_onload ev = a_onload (Eliom_xml.event_handler ev)
+      let a_onresize ev = a_onresize (Eliom_xml.event_handler ev)
+
+    end
 
   type +'a elt = 'a F.elt
+  type 'a wrap = 'a
   type +'a attrib = 'a F.attrib
   type uri = F.uri
 
@@ -241,6 +339,7 @@ module Svg = struct
   end
 
   module Printer = Xml_print.Make_typed_simple(Xml)(F)
+
 end
 
 module Html5 = struct
@@ -266,9 +365,14 @@ module Html5 = struct
       let lazy_node ?(a = []) name children =
         make_lazy (Eliom_lazy.from_fun (fun () -> (Node (name, a, Eliom_lazy.force children))))
 
+
     end
 
-    module Raw = Html5_f.Make(Xml')(Svg.D)
+    module Raw = Html5_f.Make(Xml')(Svg.D.Raw)
+    let client_attrib ?init (x : 'a Raw.attrib Eliom_lib.client_value) =
+      let crypto = make_cryptographic_safe_string () in
+      let empty_name = "" in
+      empty_name,Xml.RAClient (crypto,init,Obj.magic (Eliom_lib.client_value_server_repr x))
 
     include Raw
 
@@ -373,7 +477,7 @@ module Html5 = struct
 
   module F = struct
 
-    module Raw = Html5_f.Make(Xml)(Svg.F)
+    module Raw = Html5_f.Make(Xml)(Svg.F.Raw)
     include Raw
 
     type ('a, 'b, 'c) lazy_plus =
@@ -477,6 +581,7 @@ module Html5 = struct
   end
 
   type +'a elt = 'a F.elt
+  type 'a wrap = 'a
   type +'a attrib = 'a F.attrib
   type uri = F.uri
 
@@ -516,3 +621,7 @@ module Html5 = struct
 
 end
 
+
+(* internal modules used by reactive nodes
+   (here to escape from the interface) *)
+module Xmld = Html5.D.Xml'
