@@ -17,14 +17,15 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-(** See {% <<a_api project="ocsigenserver" | module Ocsigen_lib>> %} *)
+(** A few common functions used by Eliom.
+    See also {% <<a_api project="ocsigenserver" | module Ocsigen_lib>> %} *)
+
 include module type of Ocsigen_lib
   with type poly = Ocsigen_lib.poly
   and type yesnomaybe = Ocsigen_lib.yesnomaybe
   and type ('a, 'b) leftright = ('a, 'b) Ocsigen_lib.leftright
   and type 'a Clist.t = 'a Ocsigen_lib.Clist.t
   and type 'a Clist.node = 'a Ocsigen_lib.Clist.node
-  and type Ip_address.t = Ocsigen_lib.Ip_address.t
 
 include module type of Eliom_lib_base
   with type 'a Int64_map.t = 'a Eliom_lib_base.Int64_map.t
