@@ -318,6 +318,10 @@ let register_aux pages
           in
           (match (key_kind, attserget, attserpost) with
             | (Ocsigen_http_frame.Http_header.POST, _,
+               Eliom_common.SAtt_csrf_safe (id, scope, secure_session))
+            | (Ocsigen_http_frame.Http_header.PUT, _,
+               Eliom_common.SAtt_csrf_safe (id, scope, secure_session))
+            | (Ocsigen_http_frame.Http_header.DELETE, _,
                Eliom_common.SAtt_csrf_safe (id, scope, secure_session)) ->
               let tablereg, forsession =
                 match table with
@@ -615,7 +619,7 @@ let register_service pages
     ~get_params
     ?error_handler
     page =
-  let u = service ?https ?priority ~path ~get_params () in
+  let u = Unsafe.service ?https ?priority ~path ~get_params () in
   register pages
     ?scope
     ?options
@@ -647,7 +651,7 @@ let register_coservice pages
     ?error_handler
     page =
   let u =
-    coservice ?name
+    Unsafe.coservice ?name
       ?csrf_safe
       ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
       ?csrf_secure
@@ -684,7 +688,7 @@ let register_coservice' pages
     ?error_handler
     page =
   let u =
-    coservice'
+    Unsafe.coservice'
       ?name
       ?csrf_safe
       ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
@@ -716,8 +720,9 @@ let register_post_service pages
     ~post_params
     ?error_handler
     page_gen =
-  let u = post_service ?https ?priority
-    ~fallback:fallback ~post_params:post_params () in
+  let u = Unsafe.post_service ?https ?priority
+    ~fallback:fallback ~post_params:post_params ()
+  in
   register pages
     ?scope
     ?options
@@ -749,7 +754,7 @@ let register_post_coservice pages
     ?error_handler
     page_gen =
   let u =
-    post_coservice ?name
+    Unsafe.post_coservice ?name
       ?csrf_safe
       ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
       ?csrf_secure
@@ -786,7 +791,7 @@ let register_post_coservice' pages
     ?error_handler
     page_gen =
   let u =
-    post_coservice'
+    Unsafe.post_coservice'
       ?name
       ?csrf_safe
       ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
@@ -808,6 +813,213 @@ let register_post_coservice' pages
     ~service:u ?error_handler page_gen;
   u
 
+let register_put_service pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ?https
+    ?priority
+    ~path
+    ~get_params
+    ?error_handler
+    page_gen =
+  let u = Unsafe.put_service ?https ?priority ~path ~get_params () in
+  register pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ~service:u ?error_handler page_gen;
+  u
+
+let register_put_coservice pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ?name
+    ?csrf_safe
+    ?csrf_scope
+    ?csrf_secure
+    ?max_use
+    ?timeout
+    ?https
+    ~fallback
+    ~get_params
+    ?error_handler
+    page_gen =
+  let u =
+    Unsafe.put_coservice ?name
+      ?csrf_safe
+      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
+      ?csrf_secure
+      ?max_use ?timeout ?https
+      ~fallback ~get_params ()
+  in
+  register pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ~service:u ?error_handler page_gen;
+  u
+
+let register_put_coservice' pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ?name
+    ?csrf_safe
+    ?csrf_scope
+    ?csrf_secure
+    ?max_use
+    ?timeout
+    ?https
+    ~get_params
+    ?error_handler
+    page_gen =
+  let u =
+    Unsafe.put_coservice'
+      ?name
+      ?csrf_safe
+      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
+      ?csrf_secure
+      ?max_use
+      ?timeout
+      ?https
+      ~get_params ()
+  in
+  register pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ~service:u ?error_handler page_gen;
+  u
+
+let register_delete_service pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ?https
+    ?priority
+    ~path
+    ~get_params
+    ?error_handler
+    page_gen =
+  let u = Unsafe.delete_service ?https ?priority ~path ~get_params () in
+  register pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ~service:u ?error_handler page_gen;
+  u
+
+let register_delete_coservice pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ?name
+    ?csrf_safe
+    ?csrf_scope
+    ?csrf_secure
+    ?max_use
+    ?timeout
+    ?https
+    ~fallback
+    ~get_params
+    ?error_handler
+    page_gen =
+  let u =
+    Unsafe.delete_coservice ?name
+      ?csrf_safe
+      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
+      ?csrf_secure
+      ?max_use ?timeout ?https
+      ~fallback ~get_params () in
+  register pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ~service:u ?error_handler page_gen;
+  u
+
+let register_delete_coservice' pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ?name
+    ?csrf_safe
+    ?csrf_scope
+    ?csrf_secure
+    ?max_use
+    ?timeout
+    ?https
+    ~get_params
+    ?error_handler
+    page_gen =
+  let u =
+    Unsafe.delete_coservice'
+      ?name
+      ?csrf_safe
+      ?csrf_scope:(csrf_scope:>Eliom_common.user_scope option)
+      ?csrf_secure
+      ?max_use
+      ?timeout
+      ?https
+      ~get_params ()
+  in
+  register pages
+    ?scope
+    ?options
+    ?charset
+    ?code
+    ?content_type
+    ?headers
+    ?secure_session
+    ~service:u ?error_handler page_gen;
+  u
+
 module type REG_PARAM = "sigs/eliom_reg_param.mli"
 
 module MakeRegister(Pages : REG_PARAM) = struct
@@ -830,6 +1042,12 @@ module MakeRegister(Pages : REG_PARAM) = struct
   let register_post_service ?scope = register_post_service pages ?scope
   let register_post_coservice ?scope = register_post_coservice pages ?scope
   let register_post_coservice' ?scope = register_post_coservice' pages ?scope
+  let register_put_service ?scope = register_put_service pages ?scope
+  let register_put_coservice ?scope = register_put_coservice pages ?scope
+  let register_put_coservice' ?scope = register_put_coservice' pages ?scope
+  let register_delete_service ?scope = register_delete_service pages ?scope
+  let register_delete_coservice ?scope = register_delete_coservice pages ?scope
+  let register_delete_coservice' ?scope = register_delete_coservice' pages ?scope
 
 end
 
@@ -837,11 +1055,11 @@ module type REG_PARAM_ALPHA_RETURN =
 sig
   type ('a, 'b) page
   type 'a return
-  type ('a, 'b) result
+  type 'a result
   include "sigs/eliom_reg_param.mli"
     subst type page := ('a, 'b) page
       and type return := 'b return
-      and type result := ('a, 'b) result
+      and type result := 'a result
 end
 
 module MakeRegister_AlphaReturn(Pages : REG_PARAM_ALPHA_RETURN) = struct
@@ -849,7 +1067,7 @@ module MakeRegister_AlphaReturn(Pages : REG_PARAM_ALPHA_RETURN) = struct
   type ('a, 'b) page = ('a, 'b) Pages.page
   type options = Pages.options
   type 'b return = 'b Pages.return
-  type ('a, 'b) result = ('a, 'b) Pages.result
+  type 'a result = 'a Pages.result
 
   let pages =
     { send = Pages.send;
@@ -864,5 +1082,11 @@ module MakeRegister_AlphaReturn(Pages : REG_PARAM_ALPHA_RETURN) = struct
   let register_post_service ?scope = register_post_service pages ?scope
   let register_post_coservice ?scope = register_post_coservice pages ?scope
   let register_post_coservice' ?scope = register_post_coservice' pages ?scope
+  let register_put_service ?scope = register_put_service pages ?scope
+  let register_put_coservice ?scope = register_put_coservice pages ?scope
+  let register_put_coservice' ?scope = register_put_coservice' pages ?scope
+  let register_delete_service ?scope = register_delete_service pages ?scope
+  let register_delete_coservice ?scope = register_delete_coservice pages ?scope
+  let register_delete_coservice' ?scope = register_delete_coservice' pages ?scope
 
 end
