@@ -185,8 +185,8 @@ let reconstruct_params_
           failwith "It is not possible to have non localized parameters in suffix"
       | TJson (_, Some typ), v::l -> Deriving_Json.from_string typ v, l
       | TJson (_, None), v::l -> assert false (* client side only *)
-      | TAny _, _ ->
-          failwith "It is not possible to use any in suffix. May be try with all_suffix ?"
+      | TAny, _ ->
+        failwith "It is not possible to use any in suffix. May be try with all_suffix ?"
       | _ -> raise Eliom_common.Eliom_Wrong_parameter
   in
   let aux2 typ params =
@@ -228,6 +228,7 @@ let reconstruct_params_
             (try
                (match aux t params files pref suff with
                   | Res_ (v, l, f) -> Res_ ((Obj.magic (Some v)), l, f)
+                  | Errors_ (errs, ll, ff) when List.for_all (fun (_,s,_) -> s="") errs -> Res_ ((Obj.magic None), ll, ff)
                   | err -> err)
              with Not_found -> Res_ ((Obj.magic None), params, files))
         | TNEOption t ->
@@ -237,7 +238,7 @@ let reconstruct_params_
                     if (Obj.tag (Obj.repr v) = Obj.string_tag) && (String.length (Obj.magic v : string) = 0)  (* Is the value an empty string? *)
                     then Res_ ((Obj.magic None), l, f)
                     else Res_ ((Obj.magic (Some v)), l, f)
-                  | Errors_ ([(_,"",_)], ll, ff) -> Res_ ((Obj.magic None), ll, ff)
+                  | Errors_ (errs, ll, ff) when List.for_all (fun (_,s,_) -> s="") errs -> Res_ ((Obj.magic None), ll, ff)
                   | err -> err)
              with Not_found -> Res_ ((Obj.magic None), params, files))
         | TBool name ->
@@ -290,8 +291,11 @@ let reconstruct_params_
             (try (Res_ ((Obj.magic (float_of_string v)),l,files))
              with e -> Errors_ ([(pref^name^suff),v,e], l, files))
         | TFile name ->
-            let v,f = List.assoc_remove (pref^name^suff) files in
-            Res_ ((Obj.magic v), params, f)
+            (try
+               let v,f = List.assoc_remove (pref^name^suff) files in
+               Res_ ((Obj.magic v), params, f)
+             with e ->
+               Errors_ ([pref^name^suff,"",e], [], files))
         | TCoord name ->
             let r1 =
               let v, l = (List.assoc_remove (pref^name^suff^".x") params) in
