@@ -20,10 +20,11 @@
 
 (** Ad-hoc runtime type representation for service parameters.
 
-    See the Eliom manual for more information about {% <<a_manual
-    chapter="server-params"|service parameters>>%}.
+    {b Please read Eliom's manual before this page
+    to learn how to use
+    {% <<a_manual chapter="server-params"|service parameters>>%}. }
 
-    {% <<outline>>%}
+    {% <<outline| <<header| **Table of contents** >> >>%}
 *)
 
 (* This comments should be in manual/params.wiki.
@@ -60,7 +61,7 @@ open Ocsigen_extensions
 open Eliom_lib
 
 (** Abstract type for service parameters. See for example the
-    parameter [~get_param] of {!val:Eliom_service.service}.
+    parameter [~get_param] of {!val:Eliom_service.Http.service}.
 
     - [ 'a] is the type for the OCaml type of the
     parameter as expected by the service handler.
@@ -69,10 +70,10 @@ open Eliom_lib
     - [ 'c] is the type of the parameter name, usually an instance of
     {!Eliom_parameter.param_name}, as used by forms construction
     functions (e. g. the last parameter of
-    {!Eliom_registration.Html5.get_form}), and specialized form widget (see
+    {!Eliom_content.Html5.D.get_form}), and specialized form widget (see
     for example the section
-    {{!section:Eliom_registration.Html5.form_widgets}Form widget} of
-    {!Eliom_registration.HTML5}). )
+    {{!section:Eliom_content.Html5.D.form_widgets}Form widget} of
+    {!Eliom_content.HTML5.D}). )
 *)
 type ('a, +'b, +'c) params_type
 
@@ -389,15 +390,15 @@ val suffix_const :
       (unit, [ `WithoutSuffix ], [ `One of unit ] param_name) params_type
 
 (** marshaled OCaml values of type 'a *)
-type 'a caml
+type 'a ocaml
 
-(** [caml s] tells that the service is expecting some caml (client side)
+(** [ocaml s] tells that the service is expecting some caml (client side)
     program to send some value of type 'a, marshaled.
     As usual [s] is the name of the parameter. *)
-val caml :
+val ocaml :
   string ->
   'a Deriving_Json.t ->
-  ('a, [ `WithoutSuffix ], [ `One of 'a caml ] param_name) params_type
+  ('a, [ `WithoutSuffix ], [ `One of 'a ocaml ] param_name) params_type
 
 (** When the content type is neither URLencoded form data or multipart data,
     it is possible to get it as a stream of strings.
@@ -406,9 +407,12 @@ val caml :
     and it is not possible to create a form towards a service taking
     this kind of parameter.
 *)
+type raw_post_data =
+  ((string * string) * (string * string) list) option *
+    string Ocsigen_stream.t option
+
 val raw_post_data :
-  (((string * string) * (string * string) list) option *
-      string Ocsigen_stream.t option,
+  (raw_post_data,
    [ `WithoutSuffix ], no_param_name) params_type
 
 (** {2 Non localized parameters} *)
@@ -473,7 +477,8 @@ val get_to_and_from : ('a, 'b, 'c) params_type -> (string -> 'a) * ('a -> string
 
 (**/**)
 
-val walk_parameter_tree : [ `One of string ] param_name -> ('a, 'b, 'c) params_type
+val walk_parameter_tree :
+  [ `One of string ] param_name -> ('a, 'b, 'c) params_type
   -> ((string -> 'd) * ('d -> string)) option
 val contains_suffix : ('a, 'b, 'c) params_type -> bool option
   (* None = no suffix. The bool means : redirect_if_not_suffix *)
@@ -526,9 +531,9 @@ val nl_prod :
   ('a * 's, 'su, 'an * 'sn) params_type
 
 val remove_from_nlp :
-  (string * string) list String.Table.t ->
+  (string * 'c) list String.Table.t ->
   ('a, [< `WithSuffix | `WithoutSuffix ], 'b) params_type ->
-  (string * string) list String.Table.t
+  (string * 'c) list String.Table.t
 
 val table_of_nl_params_set : nl_params_set ->
   (string * string) list String.Table.t
