@@ -84,7 +84,7 @@ val get_header_hostname : unit -> string option
  *)
 val get_hostname : unit -> string
 
-(** returns the port of the server. 
+(** returns the port of the server.
     It is either the default port in the configuration file
     (if [<usedefaulthostname/>] is present is the configuration file),
     or the port in the Host header of the request (if present),
@@ -95,6 +95,10 @@ val get_server_port : unit -> int
 (** returns true if https is used, false if http. *)
 val get_ssl : unit -> bool
 
+(** returns the (string * float option) list corresponding
+    to accept_language HTTP header of the request. *)
+val get_accept_language : unit -> (string * float option) list
+
 (** returns the suffix of the current URL *)
 val get_suffix : unit -> Url.path option
 
@@ -131,9 +135,9 @@ val clean_request_cache : unit -> unit
 *)
 val get_link_too_old : unit -> bool
 
-(** returns the list of names of service sessions expired for the current 
+(** returns the list of names of service sessions expired for the current
     request, for browser sessions and tab sessions. *)
-val get_expired_service_sessions : 
+val get_expired_service_sessions :
   unit -> (Eliom_common.full_state_name list *
                        Eliom_common.full_state_name list)
 
@@ -167,6 +171,11 @@ val get_filesize : file_info -> int64
 (** returns the name the file had on the client when it has been sent. *)
 val get_original_filename : file_info -> string
 
+(** returns the content type sent by the browser with the file
+    (if any). *)
+val get_file_content_type :
+  file_info -> ((string * string) * (string * string) list) option
+
 (** returns the root of the site. *)
 val get_site_dir : unit -> Url.path
 
@@ -205,15 +214,15 @@ val get_initial_get_params : unit -> (string * string) list
 val get_other_get_params : unit -> (string * string) list
 
 (** returns non localized parameters in the URL. *)
-val get_nl_get_params : 
+val get_nl_get_params :
   unit -> (string * string) list String.Table.t
 
 (** returns persistent non localized parameters in the URL. *)
-val get_persistent_nl_get_params : 
+val get_persistent_nl_get_params :
   unit -> (string * string) list String.Table.t
 
 (** returns non localized POST parameters. *)
-val get_nl_post_params : 
+val get_nl_post_params :
   unit -> (string * string) list String.Table.t
 
 (** returns the parameters in the body of the HTTP request (POST parameters)
@@ -302,6 +311,9 @@ val get_csp_server_port : unit -> int
 *)
 val get_csp_ssl : unit -> bool
 
+(* Getting site configuration data *)
+val get_sitedata : unit -> Eliom_common.sitedata
+
 (**/**)
 
 val get_csp_original_full_path_sp : Eliom_common.server_params -> Url.path
@@ -312,7 +324,6 @@ val get_csp_ssl_sp : Eliom_common.server_params -> bool
 (*****************************************************************************)
 
 val get_sitedata_sp : sp:Eliom_common.server_params -> Eliom_common.sitedata
-val get_sitedata : unit -> Eliom_common.sitedata
 
 (*
 (** returns the cookie expiration date for the session,
@@ -352,11 +363,11 @@ val get_hostname_sp : Eliom_common.server_params -> string
 val get_full_url_sp : Eliom_common.server_params -> string
 
 val get_other_get_params_sp : Eliom_common.server_params -> (string * string) list
-val get_nl_get_params_sp : 
+val get_nl_get_params_sp :
   Eliom_common.server_params -> (string * string) list String.Table.t
-val get_persistent_nl_get_params_sp : 
+val get_persistent_nl_get_params_sp :
   Eliom_common.server_params -> (string * string) list String.Table.t
-val get_nl_post_params_sp : 
+val get_nl_post_params_sp :
   Eliom_common.server_params -> (string * string) list String.Table.t
 
 val get_original_full_path_sp : Eliom_common.server_params -> Url.path
@@ -369,4 +380,3 @@ val get_files_sp : Eliom_common.server_params -> (string * file_info) list Lwt.t
 val get_suffix_sp : Eliom_common.server_params -> Url.path option
 val get_request_cache_sp : Eliom_common.server_params -> Polytables.t
 val get_request_id_sp : Eliom_common.server_params -> int64
-
