@@ -19,18 +19,30 @@
  *)
 
 
-(** Creation and manipulation of Eliom services.
+(** Creation and manipulation of Eliom services. *)
 
-    See the Eliom manual for a detailed introduction to the concept of
+(**
+
+    {b See the Eliom manual for a detailed introduction to the concept of
     {% <<a_manual chapter="server-services"|Eliom services>>%}.
+    }
+
+  {% <<outline| <<header| **Table of contents** >> >>%}
+*)
+
+(**
+   The main functions to create services are in modules
+   {% <<a_api subproject="server"|module Eliom_service.Http>>%} (default),
+   {% <<a_api subproject="server"|module Eliom_service.Ocaml>>%}
+   (for services returning OCaml values) and
+   {% <<a_api subproject="server"|module Eliom_service.App>>%}
+   (for services belonging to an Eliom client-server app).
 
-    {% <<outline>> %}
  *)
 
 open Ocsigen_extensions
 
 open Eliom_lib
-open Eliom_content_core
 open Eliom_parameter
 
 
@@ -53,7 +65,7 @@ type attached_service_kind =
 
 (** The kind of a service is [`Post] when there is at least one POST
     parameters. It is [`Get] otherwise. *)
-type getpost = [ `Get | `Post ]
+type getpost = [ `Get | `Post | `Put | `Delete ]
 
 (** {4 Attached or Non-attached} *)
 
@@ -78,16 +90,26 @@ type service_kind =
 
 (** {4 Common subtypes of [service_kind] } *)
 
-(** Restriction of [service_kind] to services without POST parameters. *)
+(** Restriction of [service_kind] to GET services. *)
 type get_service_kind =
     [ `Attached of (attached_service_kind, [ `Get ]) a_s
     | `Nonattached of [ `Get ] na_s ]
 
-(** Restriction of [service_kind] to services with at least one POST parameters. *)
+(** Restriction of [service_kind] to POST services. *)
 type post_service_kind =
     [ `Attached of (attached_service_kind, [ `Post ]) a_s
     | `Nonattached of [ `Post ] na_s ]
 
+(** Restriction of [service_kind] to PUT services. *)
+type put_service_kind =
+    [ `Attached of (attached_service_kind, [ `Put ]) a_s
+    | `Nonattached of [ `Put ] na_s ]
+
+(** Restriction of [service_kind] to DELETE services. *)
+type delete_service_kind =
+    [ `Attached of (attached_service_kind, [ `Delete ]) a_s
+    | `Nonattached of [ `Delete ] na_s ]
+
 (** Restriction of [service_kind] to attached services. *)
 type attached =
     [ `Attached of (attached_service_kind, getpost) a_s ]
@@ -127,9 +149,9 @@ type registrable = [ `Registrable | `Unregistrable ]
     - the type paremeter of subtype {!suff} states the kind
       of parameters it uses: suffix or not.
     - [ 'd] is the type of GET parameters names. See {!Eliom_parameter.param_name} and
-            form generation functions (e. g. {!Eliom_registration.Html5.get_form}).
+            form generation functions (e. g. {!Eliom_content.Html5.D.get_form}).
     - [ 'e] is the type of POST parameters names. See {!Eliom_parameter.param_name} and
-            form generation functions (e. g. {!Eliom_registration.Html5.post_form}).
+            form generation functions (e. g. {!Eliom_content.Html5.D.post_form}).
     - the type parameter of subtype {!registrable} tells if it is possible to
       register a handler on this service.
     - [ 'f] is an information on what the service returns.
@@ -139,313 +161,79 @@ type ('a,'b,+'c,+'d,+'e,+'f,+'g,+'h) service
 constraint 'd = [< suff ]
 constraint 'g = [< registrable ]
 
+(** Types of groups of services. *)
+
+type http_service = [ `Http ]
+type appl_service = [ `Appl ]
+type 'a ocaml_service
+
+(** The type [non_ocaml_service] is used as phantom type parameters for
+    the {!Eliom_registration.kind}. It used to type functions that operates
+    over service that do not returns OCaml values, like
+    {!appl_self_redirect}. *)
+type non_ocaml_service = [ appl_service | http_service ]
+
+(** Helper for typing OCaml services.
+    In some cases, you may need to write the return type of the
+    service manually. Instead of writing the full type of the service,
+    (which may be huge), add a type constraint for parameter [?rt] of service
+    creation functions
+    (like <<a_api subproject="server"|fun Eliom_service.Http.service>>),
+    using the following value.
+
+*)
+type 'rt rt
+val rt : 'rt rt
 
 (***** Static dir and actions do not depend on the type of pages ******)
 
-(** {2 Registration of named modules}
+(** {2 Registration of services} *)
 
+
+(** Default module for creating services *)
+module Http : "sigs/eliom_service_with_external.mli"
+  subst type returnB := [> http_service ]
+  and type returnT := [< non_ocaml_service ]
+
+(** Module for creating services returning applications *)
+module App : "sigs/eliom_service.mli"
+  subst type returnB := [> appl_service ]
+  and type returnT := [< non_ocaml_service ]
+
+(** Module for creating services that return OCaml values *)
+module Ocaml : "sigs/eliom_service_with_external.mli"
+  subst type returnB := 'rt ocaml_service
+  and type returnT := 'rt ocaml_service
+
+(** Module for creating services without specifying the return type *)
+module Unsafe : "sigs/eliom_service_with_external.mli"
+  subst type returnB := 'returnB
+  and type returnT := 'returnT
+
+
+(** {2 Static loading of Eliom modules} *)
+
+(**
     This functionality allows one to register initialization functions for
     Eliom modules which will be executed when the corresponding module
     is loaded in [ocsigenserver.conf].
+    If the module is loaded dynamically, you probably don't need this.
+   But if the module is linked statically, some computations,
+   like service registrations must be delayed.
 
 *)
 
 (** The function [register_eliom_module mod f] is used to register the
     initialization function [f] to be executed when then module [mod]
-    is "loaded" by Ocsigen server. The module [mod] could either be a
+    is loaded by Ocsigen server. The module [mod] could either be a
     dynamically loaded module or linked statically into the server: in
     each case, the [f] function will be invoked when the module is
-    initialized in the configuration file using [<eliom name="name">
-    ... </eliom>]. If [register_eliom_module] is called twice with the
+    initialized in the configuration file using [<eliommodule ...>
+    ... </eliommodule>]. If [register_eliom_module] is called twice with the
     same module name, the second initialization function will replace
     the previous one. *)
 val register_eliom_module : string -> (unit -> unit) -> unit
 
-(** {2 Definitions of services}
-
-    {e Warning: These functions must be called when the site
-    information is available, that is, either during a request or
-    during the initialisation phase of the site.  Otherwise, it will
-    raise the exception
-    {!Eliom_common.Eliom_site_information_not_available}.  If you are
-    using static linking, you must delay the call to this function
-    until the configuration file is read, using
-    {!Eliom_service.register_eliom_module}. Otherwise you will also
-    get this exception.}
-*)
-
-(** {3 Main services} *)
-
-(** The function [service ~path ~get_params ()] creates a {!service}
-    associated to the path [path] and taking [get_params] as GET
-    parameters.
-
-    If the optional parameter [~https:true] is given, all links
-    towards that service will use https. By default, links will keep
-    the current protocol.
-
-    The optional parameter [~priority] allows one to change the priority
-    order between service that shares the same path. The default
-    priority is 0 ; if you want the service to be tried before
-    (resp. after) other services, put a higher (resp. lower) priority.
-
-    If the optional parameter [~keep_nl_params:`Persistent]
-    (resp. [~keep_nl_params:`All]) is given, all links towards that
-    service will keep persistent (resp. all) non localized GET
-    parameters of the current service. The default is [`None]. See the
-    eliom manual for more information about {% <<a_manual
-    chapter="server-params" fragment="nonlocalizedparameters"|non localized
-    parameters>>%}.
-*)
-val service :
-  ?https:bool ->
-  path:Url.path ->
-  ?keep_nl_params:[ `All | `Persistent | `None ] ->
-  ?priority:int ->
-  get_params:('get, [< suff ] as 'tipo,'gn) params_type ->
-  unit ->
-  ('get,unit,
-   [> `Attached of
-       ([> `Internal of [> `Service ] ], [> `Get ]) a_s ],
-   'tipo,'gn,
-   unit, [> `Registrable ], 'return) service
-
-
-
-(** The function [external_service ~prefix ~path ~get_params ()]
-    creates a service for an external web site, that will use GET
-    method and requires [get_params] as parameters. This allows one to
-    creates links or forms towards other Web sites using Eliom's
-    syntax.
-
-    The parameter labelled [~path] is the URL path. Each element of
-    the list will be URL-encoded.
-
-    The parameter labelled [~prefix] contains all what you want to put
-    before the path. It usually starts with "http://" plus the name of
-    the server. The prefix is not URL encoded.
-
-    The whole URL is constructed from the prefix, the path and GET
-    parameters. Hence, an empty prefix can be used to make a link to
-    another site of the same server.
-
-    See {!val:service} for a description of the optional
-    [~keep_nl_params] parameter.
-*)
-val external_service :
-  prefix: string ->
-  path:Url.path ->
-  ?keep_nl_params:[ `All | `Persistent | `None ] ->
-  get_params:('get, [< suff ] as 'tipo, 'gn) params_type ->
-  unit ->
-  ('get, unit, [> `Attached of ([> `External ], [> `Get ]) a_s ], 'tipo,
-   'gn, unit, [> `Unregistrable ], 'return) service
-
-
-(** Same as {!external_service} but with POST method. *)
-val external_post_service :
-  prefix: string ->
-  path:Url.path ->
-  ?keep_nl_params:[ `All | `Persistent | `None ] ->
-  get_params:('get, [< suff ] as 'tipo, 'gn) params_type ->
-  post_params:('post, [ `WithoutSuffix ], 'pn) params_type ->
-  unit ->
-  ('get, 'post, [> `Attached of ([> `External ], [> `Post ]) a_s ], 'tipo,
-   'gn, 'pn, [> `Unregistrable ], 'return) service
-
-
-(** The function [post_service ~fallback ~post_params ()] creates a
-    service that takes [post_params] as POST parameters and share the
-    same path and GET parameters than the service [fallback].
-
-    POST parameters couldn't contain a suffix parameter.
-
-    The service [fallback] should be an (internal) attached service
-    without POST parameters ; it couldn't be a preapplied service.
-    This argument enforces the creation of an equivalent service (
-    i.e. a service with the same path and GET parameters ) to be
-    served when POST parameters are missing. Thus, the user cannot put
-    a bookmark on a page that does not exist.
-
-    See {!service} for a description of optional [~https],
-    [~keep_nl_params] and [~priority] parameters .
-*)
-val post_service :
-  ?https:bool ->
-  fallback: ('get, unit,
-             [`Attached of
-                 ([`Internal of
-                     ([ `Service | `Coservice ] as 'kind) ], [`Get]) a_s ],
-             [< suff] as 'tipo, 'gn, unit,
-             [< `Registrable ], 'return1) service ->
-  ?keep_nl_params:[ `All | `Persistent | `None ] ->
-  ?priority:int ->
-  post_params: ('post, [`WithoutSuffix], 'pn) params_type ->
-  unit ->
-  ('get, 'post, [> `Attached of
-      ([> `Internal of 'kind ], [> `Post]) a_s ],
-   'tipo, 'gn, 'pn, [> `Registrable ], 'return2) service
-
-
-(** {3 Attached coservices} *)
-
-(** The function [coservice ~fallback ~get_params] creates an {%
-    <<a_manual chapter="server-services" fragment="attached_coservices"|attached
-    coservice>>%} at the same path than the service [fallback] and
-    taking [get_params] as GET parameters.
-
-    GET parameters of [coservice] couldn't contain a suffix
-    parameter.
-
-    The service [fallback] should be an (internal) attached service
-    without any GET or POST parameters ; it could be a preapplied
-    service.
-
-    The optional [~name] parameter Coservices can be named if the [?name] optional parameter is present
-    or anonymous (in that case, a coservice number will be generated).
-
-    The optional [~timeout] parameter specifies a timeout (in
-    seconds) after which the coservice will disappear. This amount of
-    time is computed from the creation or from the last call to the
-    service. The default is "no timeout". The optional [~max_use]
-    parameter specifies that the service can be used only a fixed
-    number of times. The default is "no limitation".
-
-    If the optional [~csrf_safe] parameter is [true], it will create a
-    {% <<a_manual chapter="server-security" fragment="csrf"|"CSRF-safe"
-    service>>%}. In that case the [~name] parameter is ignored. The
-    default is [false].
-
-    The [~csrf_scope] and [~csrf_secure], if present, should
-    respectively correspond to the [~scope] and [~secure] parameters
-    that will be given to the associated [register]
-    function. Otherwise the registration will fail with
-    {Eliom_service.Wrong_session_table_for_CSRF_safe_coservice}. See
-    {!Eliom_registration.Html5.register},
-    {!Eliom_registration.App.register} or any other
-    {!Eliom_registration}[.*.register] functions for a description of those
-    parameters.
-
-    See {!service} for a description of the optional [~https] and
-    [~keep_nl_params] parameters .
-*)
-val coservice :
-  ?name: string ->
-  ?csrf_safe: bool ->
-  ?csrf_scope: [< Eliom_common.user_scope] ->
-  ?csrf_secure: bool ->
-  ?max_use:int ->
-  ?timeout:float ->
-  ?https:bool ->
-  fallback:
-    (unit, unit, [ `Attached of ([ `Internal of [ `Service ] ], [`Get]) a_s ],
-     [ `WithoutSuffix ] as 'tipo,
-     unit, unit, [< registrable ], 'return1) service ->
-  ?keep_nl_params:[ `All | `Persistent | `None ] ->
-  get_params:
-    ('get,[`WithoutSuffix],'gn) params_type ->
-  unit ->
-  ('get,unit,[> `Attached of
-      ([> `Internal of [> `Coservice] ], [> `Get]) a_s ],
-   'tipo, 'gn, unit,
-   [> `Registrable ], 'return2) service
-
-(** The function [post_coservice ~fallback ~get_params] creates an {%
-    <<a_manual chapter="server-services" fragment="attached_coservices"|attached
-    coservice>>%} with the same path and GET parameters than the
-    service [fallback] and taking [post_params] as POST
-    parameters.
-
-    POST parameters couldn't contain a suffix parameter.
-
-    The service [fallback] should be an (internal) attached service or
-    coservice without any POST parameters ; it could be a preapplied
-    service.
-
-    See {!coservice} for a description of optional parameters. *)
-val post_coservice :
-  ?name: string ->
-  ?csrf_safe: bool ->
-  ?csrf_scope: [< Eliom_common.user_scope] ->
-  ?csrf_secure: bool ->
-  ?max_use:int ->
-  ?timeout:float ->
-  ?https:bool ->
-  fallback: ('get, unit, [ `Attached of
-                             ([`Internal of [<`Service | `Coservice] ],
-                              [`Get]) a_s ],
-             [< suff ] as 'tipo,
-             'gn, unit, [< `Registrable ], 'return1) service ->
-  ?keep_nl_params:[ `All | `Persistent | `None ] ->
-  post_params: ('post, [`WithoutSuffix], 'pn) params_type ->
-  unit ->
-  ('get, 'post,
-   [> `Attached of
-      ([> `Internal of [> `Coservice] ], [> `Post]) a_s ],
-   'tipo, 'gn, 'pn, [> `Registrable ], 'return2) service
-
-(** {3 Non attached coservices} *)
-
-(** The function [coservice' ~get_param] creates a {% <<a_manual
-    chapter="server-services" fragment="non-attached_coservices"|non-attached
-    coservice>>%} taking [get_params] as extra GET parameters.
-
-    GET parameters of [coservice'] couldn't contain a suffix
-    parameter.
-
-    See {!service} for a description of the optional [~https] and
-    [~keep_nl_params] parameters ; see {!coservice} for others
-    optional parameters.
-*)
-val coservice' :
-  ?name:string ->
-  ?csrf_safe: bool ->
-  ?csrf_scope: [< Eliom_common.user_scope] ->
-  ?csrf_secure: bool ->
-  ?max_use:int ->
-  ?timeout:float ->
-  ?https:bool ->
-  ?keep_nl_params:[ `All | `Persistent | `None ] ->
-  get_params:
-    ('get, [`WithoutSuffix], 'gn) params_type ->
-  unit ->
-  ('get, unit, [> `Nonattached of [> `Get] na_s ],
-   [`WithoutSuffix], 'gn, unit, [> `Registrable ], 'return) service
-
-
-(** The function [post_coservice' ~get_param] creates a {% <<a_manual
-    chapter="server-services" fragment="non-attached_coservices"|non-attached
-    coservice>>%} taking [post_params] as POST parameters.
-
-    POST parameters couldn't contain a suffix parameter.
-
-    If the optional parameter [~keep_get_na_params] is [false], GET
-    non-attached parameters of the current page won't be kept in the
-    URL (if any) when you create a POST form to this coservice. The
-    default is true.
-
-    See {!service} for a description of the optional [~https] and
-    [~keep_nl_params] parameters ; see {!coservice} for others
-    optional parameters.
-*)
-val post_coservice' :
-  ?name:string ->
-  ?csrf_safe: bool ->
-  ?csrf_scope: [< Eliom_common.user_scope] ->
-  ?csrf_secure: bool ->
-  ?max_use:int ->
-  ?timeout:float ->
-  ?https:bool ->
-  ?keep_nl_params:[ `All | `Persistent | `None ] ->
-  ?keep_get_na_params:bool ->
-  post_params: ('post, [`WithoutSuffix], 'pn) params_type ->
-  unit ->
-  (unit, 'post,
-   [> `Nonattached of [> `Post ] na_s ],
-   [ `WithoutSuffix ], unit, 'pn, [> `Registrable ], 'return) service
-
-
-
 (** {2 Predefined services} *)
 
 (** {3 Static files} *)
@@ -460,7 +248,8 @@ val static_dir :
   (string list, unit, [> `Attached of
       ([> `Internal of [> `Service ] ], [> `Get]) a_s ],
    [ `WithSuffix ],
-   [ `One of string list ] param_name, unit, [> `Unregistrable ], 'return)
+   [ `One of string list ] param_name, unit, [> `Unregistrable ],
+   [> http_service ])
     service
 
 (** Same as {!static_dir} but forcing https link. *)
@@ -469,7 +258,8 @@ val https_static_dir :
   (string list, unit, [> `Attached of
       ([> `Internal of [> `Service ] ], [> `Get]) a_s ],
    [ `WithSuffix ],
-   [ `One of string list ] param_name, unit, [> `Unregistrable ], 'return)
+   [ `One of string list ] param_name, unit, [> `Unregistrable ],
+   [> http_service ])
     service
 
 (** Like [static_dir], but allows one to put GET parameters *)
@@ -481,7 +271,8 @@ val static_dir_with_params :
    [> `Attached of
       ([> `Internal of [> `Service ] ], [> `Get]) a_s ],
    [ `WithSuffix ],
-   [ `One of string list ] param_name *'an, unit, [> `Unregistrable ], 'return)
+   [ `One of string list ] param_name *'an, unit, [> `Unregistrable ],
+   [> http_service ])
     service
 
 (** Same as {!static_dir_with_params} but forcing https link. *)
@@ -493,7 +284,8 @@ val https_static_dir_with_params :
    [> `Attached of
       ([> `Internal of [> `Service ] ], [> `Get]) a_s ],
    [ `WithSuffix ],
-   [ `One of string list ] param_name *'an, unit, [> `Unregistrable ], 'return)
+   [ `One of string list ] param_name *'an, unit, [> `Unregistrable ],
+   [> http_service ])
     service
 
 
@@ -510,30 +302,30 @@ val https_static_dir_with_params :
     it with <a> links, not only forms.  It does not keep non attached
     GET parameters.  *)
 val void_coservice' :
-  (unit, unit, [> `Nonattached of 'a na_s ],
+  (unit, unit, [> `Nonattached of [> `Get ] na_s ],
    [ `WithoutSuffix ],
-   unit, unit, [> `Unregistrable ], 'return)
+   unit, unit, [> `Unregistrable ], [> non_ocaml_service ])
   service
 
 (** Same as {!void_coservice'} but forcing https. *)
 val https_void_coservice' :
-  (unit, unit, [> `Nonattached of 'a na_s ],
+  (unit, unit, [> `Nonattached of [> `Get ] na_s ],
    [ `WithoutSuffix ],
-   unit, unit, [> `Unregistrable ], 'return)
+   unit, unit, [> `Unregistrable ], [> non_ocaml_service ])
   service
 
 (** Same as {!void_coservice'} but keeps non attached GET parameters. *)
 val void_hidden_coservice' :
-  (unit, unit, [> `Nonattached of 'a na_s ],
+  (unit, unit, [> `Nonattached of [> `Get ] na_s ],
    [ `WithoutSuffix ],
-   unit, unit, [> `Unregistrable ], 'return)
+   unit, unit, [> `Unregistrable ], [> non_ocaml_service ])
   service
 
 (** Same as {!void_hidden_coservice'} but forcing https. *)
 val https_void_hidden_coservice' :
-  (unit, unit, [> `Nonattached of 'a na_s ],
+  (unit, unit, [> `Nonattached of [> `Get ] na_s ],
    [ `WithoutSuffix ],
-   unit, unit, [> `Unregistrable ], 'return)
+   unit, unit, [> `Unregistrable ], [> non_ocaml_service ])
   service
 
 
@@ -552,6 +344,21 @@ val preapply :
   (unit, 'b, 'c,
    [ `WithoutSuffix ], unit, 'f, [> `Unregistrable ], 'return) service
 
+(** [attach_coservice' ~fallback ~service] attaches the non-attached
+    coservice [service] on the URL of [fallback]. This allows to
+    create a link to a non-attached coservice but with another URL
+    than the current one. It is not possible to register something
+    on the service returned by this function. *)
+val attach_coservice' :
+  fallback:(unit, unit, [< `Attached of ([< `Internal of 'sc1 ],
+                                         [< `Get ]) a_s ],
+	   [< suff ], unit, unit, 'rg1, 'return1) service ->
+  service: ('get, 'post, [< `Nonattached of 'gp na_s ],
+            [< `WithoutSuffix] as 'sf, 'gn, 'pn, 'rg2, 'return) service ->
+  ('get, 'post, [> `Attached of ([> `Internal of [> `Coservice ] ],
+                                 'gp) a_s ],
+   'sf, 'gn, 'pn, [< registrable > `Unregistrable ], 'return) service
+
 (** The function [add_non_localized_get_parameters ~params ~service]
     Adds non localized GET parameters [params] to [service]. See the
     Eliom manual for more information about {% <<a_manual
@@ -579,7 +386,7 @@ val add_non_localized_post_parameters :
 val unregister :
   ?scope:[< Eliom_common.scope ] ->
   ?secure:bool ->
-  ('a, 'b, [< `Attached of ([> `Internal of 'c ], [< `Get | `Post ]) a_s
+  ('a, 'b, [< `Attached of ([> `Internal of 'c ], [< `Get | `Post | `Put | `Delete ]) a_s
 	   | `Nonattached of 'd na_s ], 'e, 'f, 'g, 'h, 'return) service ->
   unit
 
@@ -591,7 +398,7 @@ val get_get_or_post :
    | `Nonattached of [< getpost ] na_s ], 'd, 'e, 'f, 'g, 'h) service ->
   getpost
 val get_kind_ : ('a, 'b, 'c, 'd, 'e, 'f, 'g, 'return) service -> 'c
-val get_or_post_ : ('a, [< `Get | `Post ]) a_s ->
+val get_or_post_ : ('a, [< `Get | `Post | `Put | `Delete ]) a_s ->
   Ocsigen_http_frame.Http_header.http_method
 val get_pre_applied_parameters_ : ('a, 'b, 'c, 'd, 'e, 'f, 'g, 'return) service ->
   (string * string) list String.Table.t *
@@ -608,7 +415,7 @@ val get_get_name_ : ('a, 'b) a_s -> Eliom_common.att_key_serv
 val get_post_name_ : ('a, 'b) a_s -> Eliom_common.att_key_serv
 val get_redirect_suffix_ : ('a, 'b) a_s -> bool
 val get_na_name_ : 'a na_s -> Eliom_common.na_key_serv
-val get_na_kind_ : 'a na_s -> [ `Get | `Post of bool ]
+val get_na_kind_ : [< getpost ] na_s -> [ `Get | `Post of bool | `Put of bool | `Delete of bool ]
 val get_max_use_ : ('a, 'b, 'c, 'd, 'e, 'f, 'g, 'return) service -> int option
 val get_timeout_ : ('a, 'b, 'c, 'd, 'e, 'f, 'g, 'return) service -> float option
 val get_https : ('a, 'b, 'c, 'd, 'e, 'f, 'g, 'return) service -> bool
