@@ -250,7 +250,7 @@ let set_persistent_data_state_timeout ~cookie_scope ?secure t =
     ~cookie_scope ~secure () in
   let tor = c.Eliom_common.pc_timeout in
   return
-      (match t with
+    (match t with
       | None -> tor := Eliom_common.TNone
       | Some t -> tor := Eliom_common.TSome t)
 
@@ -386,11 +386,11 @@ type 'a state_data =
 
 let set_service_session_group
     ?set_max ?(scope = Eliom_common.default_session_scope)
-    ?secure session_group =
+    ?(secure = false) session_group =
   let c =
     Eliommod_sersess.find_or_create_service_cookie
       ~set_session_group:session_group
-      ~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure ()
+      ~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure:(Some secure) ()
   in
   match set_max with
     | None -> ()
@@ -399,13 +399,14 @@ let set_service_session_group
           c.Eliom_common.sc_session_group_node m
 
 let unset_service_session_group
-    ?set_max ?(scope = Eliom_common.default_session_scope) ?secure () =
+    ?set_max ?(scope = Eliom_common.default_session_scope)
+    ?(secure = false) () =
   try
     let sp = Eliom_common.get_sp () in
     let sitedata = Eliom_request_info.get_sitedata_sp ~sp in
     let c =
       Eliommod_sersess.find_service_cookie_only
-         ~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure ~sp ()
+         ~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure:(Some secure) ~sp ()
     in
     let n =
       Eliommod_sessiongroups.make_full_group_name
@@ -426,18 +427,19 @@ let unset_service_session_group
     (* Now we want to close the session if it has not data inside
        and no tab sessions *)
     close_service_state_if_empty
-      ~scope:(scope:>Eliom_common.user_scope) ?secure ()
+      ~scope:(scope:>Eliom_common.user_scope) ~secure ()
 
   with
     | Not_found
     | Eliom_common.Eliom_Session_expired -> ()
 
 let get_service_session_group
-    ?(scope = Eliom_common.default_session_scope) ?secure () =
+    ?(scope = Eliom_common.default_session_scope)
+    ?(secure = false) () =
   try
     let c =
       Eliommod_sersess.find_service_cookie_only
-	~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure ()
+	~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure:(Some secure) ()
     in
     match !(c.Eliom_common.sc_session_group) with
       | _, _, Right _ -> None
@@ -447,11 +449,11 @@ let get_service_session_group
     | Eliom_common.Eliom_Session_expired -> None
 
 let get_service_session_group_size
-    ?(scope = Eliom_common.default_session_scope) ?secure () =
+    ?(scope = Eliom_common.default_session_scope) ?(secure = false) () =
   try
     let c =
       Eliommod_sersess.find_service_cookie_only
-	~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure ()
+	~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure:(Some secure) ()
     in
     match !(c.Eliom_common.sc_session_group) with
       | _, _, Right _ -> None
@@ -463,12 +465,12 @@ let get_service_session_group_size
 
 let set_volatile_data_session_group
     ?set_max ?(scope = Eliom_common.default_session_scope)
-    ?secure session_group =
+    ?(secure = false) session_group =
   let c =
     Eliommod_datasess.find_or_create_data_cookie
       ~set_session_group:session_group
       ~cookie_scope:(scope:>Eliom_common.cookie_scope)
-      ~secure ()
+      ~secure:(Some secure) ()
   in
   match set_max with
     | None -> ()
@@ -477,14 +479,15 @@ let set_volatile_data_session_group
           c.Eliom_common.dc_session_group_node m
 
 let unset_volatile_data_session_group ?set_max
-    ?(scope = Eliom_common.default_session_scope) ?secure () =
+    ?(scope = Eliom_common.default_session_scope)
+    ?(secure = false) () =
   try
     let sp = Eliom_common.get_sp () in
     let sitedata = Eliom_request_info.get_sitedata_sp ~sp in
     let c =
       Eliommod_datasess.find_data_cookie_only
         ~cookie_scope:(scope:>Eliom_common.cookie_scope)
-	~secure ~sp ()
+	~secure:(Some secure) ~sp ()
     in
     let n =
       Eliommod_sessiongroups.make_full_group_name
@@ -504,18 +507,19 @@ let unset_volatile_data_session_group ?set_max
     (* Now we want to close the session if it has not data inside
        and no tab sessions *)
     close_volatile_state_if_empty
-      ~scope:(scope:>Eliom_common.user_scope) ?secure ()
+      ~scope:(scope:>Eliom_common.user_scope) ~secure ()
 
   with
     | Not_found
     | Eliom_common.Eliom_Session_expired -> ()
 
 let get_volatile_data_session_group
-    ?(scope =Eliom_common.default_session_scope) ?secure () =
+    ?(scope =Eliom_common.default_session_scope)
+    ?(secure = false) () =
   try
     let c =
       Eliommod_datasess.find_data_cookie_only
-        ~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure ()
+        ~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure:(Some secure) ()
     in
     match !(c.Eliom_common.dc_session_group) with
       | _, _, Right _ -> None
@@ -525,11 +529,12 @@ let get_volatile_data_session_group
     | Eliom_common.Eliom_Session_expired -> None
 
 let get_volatile_data_session_group_size
-    ?(scope = Eliom_common.default_session_scope) ?secure () =
+    ?(scope = Eliom_common.default_session_scope)
+    ?(secure = false) () =
   try
     let c =
       Eliommod_datasess.find_data_cookie_only
-	~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure ()
+	~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure:(Some secure) ()
     in
     match !(c.Eliom_common.dc_session_group) with
       | _, _, Right _ -> None
@@ -540,11 +545,12 @@ let get_volatile_data_session_group_size
     | Eliom_common.Eliom_Session_expired -> None
 
 let set_persistent_data_session_group ?set_max
-    ?(scope = Eliom_common.default_session_scope) ?secure n =
+    ?(scope = Eliom_common.default_session_scope)
+    ?(secure = false) n =
   let sp = Eliom_common.get_sp () in
   let sitedata = Eliom_request_info.get_sitedata_sp ~sp in
   lwt c = Eliommod_persess.find_or_create_persistent_cookie
-    ~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure ~sp () in
+    ~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure:(Some secure) ~sp () in
   let n =
     Eliommod_sessiongroups.make_persistent_full_group_name
       ~cookie_level:`Session
@@ -564,30 +570,30 @@ let set_persistent_data_session_group ?set_max
 
 let unset_persistent_data_session_group
     ?(scope = Eliom_common.default_session_scope)
-    ?secure () =
+    ?(secure = false) () =
   let sp = Eliom_common.get_sp () in
   let sitedata = Eliom_request_info.get_sitedata_sp ~sp in
   try_lwt
     lwt c = Eliommod_persess.find_persistent_cookie_only
       ~cookie_scope:(scope:>Eliom_common.cookie_scope)
-      ~secure ~sp () in
+      ~secure:(Some secure) ~sp () in
     let grp = c.Eliom_common.pc_session_group in
     lwt () = Eliommod_sessiongroups.Pers.remove
       sitedata c.Eliom_common.pc_value !grp in
     grp := None;
 
     close_persistent_state_if_empty
-      ~scope:(scope:>Eliom_common.user_scope) ?secure ()
+      ~scope:(scope:>Eliom_common.user_scope) ~secure ()
   with
     | Not_found
     | Eliom_common.Eliom_Session_expired -> Lwt.return ()
 
 let get_persistent_data_session_group
     ?(scope = Eliom_common.default_session_scope)
-    ?secure () =
+    ?(secure = false) () =
   try_lwt
     lwt c = Eliommod_persess.find_persistent_cookie_only
-      ~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure () in
+      ~cookie_scope:(scope:>Eliom_common.cookie_scope) ~secure:(Some secure) () in
     Lwt.return (match !(c.Eliom_common.pc_session_group) with
       | None -> None
       | Some v ->
@@ -739,10 +745,10 @@ let set_ipv6_subnet_mask ?(override_configfile = false) n =
 
 
 
-let set_max_service_states_for_group_or_subnet ~scope ?secure m =
+let set_max_service_states_for_group_or_subnet ~scope ?(secure = false) m =
   let cookie_scope = Eliom_common.cookie_scope_of_user_scope scope in
   let c =
-    Eliommod_sersess.find_or_create_service_cookie ~secure ~cookie_scope ()
+    Eliommod_sersess.find_or_create_service_cookie ~secure:(Some secure) ~cookie_scope ()
   in
   match scope with
     | `Session_group _ ->
@@ -756,10 +762,10 @@ let set_max_service_states_for_group_or_subnet ~scope ?secure m =
       Eliommod_sessiongroups.Data.set_max c.Eliom_common.sc_session_group_node m
 
 let set_max_volatile_data_states_for_group_or_subnet
-    ~scope ?secure m =
+    ~scope ?(secure = false) m =
   let cookie_scope = Eliom_common.cookie_scope_of_user_scope scope in
   let c =
-    Eliommod_datasess.find_or_create_data_cookie ~cookie_scope ~secure ()
+    Eliommod_datasess.find_or_create_data_cookie ~cookie_scope ~secure:(Some secure) ()
   in
   match scope with
     | `Session_group _ ->
@@ -773,11 +779,11 @@ let set_max_volatile_data_states_for_group_or_subnet
       Eliommod_sessiongroups.Data.set_max c.Eliom_common.dc_session_group_node m
 
 let set_max_volatile_states_for_group_or_subnet
-    ~scope ?secure m =
+    ~scope ?(secure = false) m =
   set_max_service_states_for_group_or_subnet
-    ~scope ?secure m;
+    ~scope ~secure m;
   set_max_volatile_data_states_for_group_or_subnet
-    ~scope ?secure m
+    ~scope ~secure m
 
 (*VVV No version for persistent sessions? Why? *)
 
@@ -1298,25 +1304,25 @@ module Ext = struct
 
 
   let fold_sub_states_aux_aux
+      ?(sitedata =
+          Eliom_request_info.find_sitedata "Eliom_state (state iterator)")
       ~state:((s, k, id) : ([< `Session_group | `Session ],
                             [< `Pers | `Data | `Service ]) state) f =
-    (* id is the session cookie value or the group name *)
-    let sp = Eliom_common.get_sp () in
-    let sitedata = Eliom_request_info.get_sitedata_sp ~sp in
-    let reduce_scope = function
-      | `Session_group n -> `Session n
-      | `Session n -> `Client_process n
-      | `Client_process _ -> failwith "fold_sub_states"
-    in
-    let reduce_level = function
-      | `Session_group n -> `Session
-      | `Session n -> `Client_process
-      | `Client_process _ -> failwith "fold_sub_states"
-    in
-    let sub_states_level = reduce_level s in
-    let sub_states_scope = reduce_scope s in
-    let f a v = f a (sub_states_scope, k, v) in
-    (sitedata, sub_states_level, id, f)
+      (* id is the session cookie value or the group name *)
+      let reduce_scope = function
+        | `Session_group n -> `Session n
+        | `Session n -> `Client_process n
+        | `Client_process _ -> failwith "fold_sub_states"
+      in
+      let reduce_level = function
+        | `Session_group n -> `Session
+        | `Session n -> `Client_process
+        | `Client_process _ -> failwith "fold_sub_states"
+      in
+      let sub_states_level = reduce_level s in
+      let sub_states_scope = reduce_scope s in
+      let f a v = f a (sub_states_scope, k, v) in
+      (sitedata, sub_states_level, id, f)
 
   let fold_sub_states_aux
       fold return (sitedata, sub_states_level, id, f) e = function
@@ -1336,19 +1342,18 @@ module Ext = struct
        with Not_found -> return e)
     | _ -> failwith "fold_sub_states_aux"
 
-  let fold_volatile_sub_states
+  let fold_volatile_sub_states ?sitedata
       ~(state : Eliom_common.user_scope * [> `Data | `Service ] * string)
       f e =
     let state' = (state :> ('aa, 'bb) state) in
     let (sitedata, sub_states_level, id, f) as a =
-      fold_sub_states_aux_aux ~state:state' f
+      fold_sub_states_aux_aux ?sitedata ~state:state' f
     in
     fold_sub_states_aux Ocsigen_cache.Dlist.fold Ocsigen_lib.id a e state
 
-  let fold_sub_states ~state f e =
+  let fold_sub_states ?sitedata ~state f e =
     let (sitedata, sub_states_level, id, f) as a =
-      fold_sub_states_aux_aux ~state f
-    in
+      fold_sub_states_aux_aux ?sitedata ~state f in
     match state with
       | (_, `Pers, _) ->
         (Eliommod_sessiongroups.Pers.find
@@ -1359,13 +1364,15 @@ module Ext = struct
       | _ ->
         fold_sub_states_aux Ocsigen_cache.Dlist.lwt_fold Lwt.return a e state
 
-  let iter_volatile_sub_states ~state f =
-    fold_volatile_sub_states ~state (fun () -> f) ()
+  let iter_volatile_sub_states ?sitedata ~state f =
+    fold_volatile_sub_states ?sitedata ~state (fun () -> f) ()
 
-  let iter_sub_states ~state f =
-    fold_sub_states ~state (fun () -> f) ()
+  let iter_sub_states ?sitedata ~state f =
+    fold_sub_states ?sitedata ~state (fun () -> f) ()
 
 
+  exception Wrong_scope
+
   module Low_level = struct
 
     (* We have a dynamic scope checking here.
@@ -1376,7 +1383,7 @@ module Ext = struct
 
     let check_scopes table_scope state_scope =
       if table_scope <> state_scope
-      then failwith "wrong scope"
+      then raise Wrong_scope
 
     let lwt_check_scopes a b =
       try check_scopes a b; Lwt.return ()
@@ -1385,7 +1392,7 @@ module Ext = struct
     let check_group_scope s =
       match s with
         | `Session_group _ -> ()
-        | _ -> failwith "wrong scope"
+        | _ -> raise Wrong_scope
 
     let lwt_check_group_scope a =
       try check_group_scope a; Lwt.return ()
