@@ -17,11 +17,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-(**
+(** Storing server-side values for your applications or sessions. *)
 
-Manage server-side state.
+(** {b Please read the
+    {% <<a_manual chapter="server-state" | Eliom manual >>%}
+    before this page to learn how server side state works. }
 
-{% <<outline>>%}
+{% <<outline| <<header| **Table of contents** >> >>%}
 
 *)
 
@@ -34,7 +36,7 @@ open Ocsigen_extensions
 
 (** {3 Closing sessions, removing state data and services} *)
 
-(** Delete server-side (vatile and persistent) state data and services
+(** Delete server-side (volatile and persistent) state data and services
     for a session,
     a group of sessions, a client process or a request.
 
@@ -175,9 +177,8 @@ val unset_cookie :
     configuration for the group or default configuration if the group
     does not exist.
 
-    If [~secure] is false when the protocol is https, it will affect
-    the unsecure session. Otherwise, il will affect the secure session in
-    https, the unsecure one in http.
+    If [~secure] is true, it will affect the secure session (secure cookies),
+    otherwise (default), the unsecure one (behavior change in Eliom 4).
 *)
 val set_service_session_group :
   ?set_max: int ->
@@ -389,11 +390,11 @@ val set_default_max_volatile_tab_sessions_per_group :
 
 (** Sets the mask for subnet (IPV4). *)
 val set_ipv4_subnet_mask :
-  ?override_configfile:bool -> int32 -> unit
+  ?override_configfile:bool -> int -> unit
 
 (** Sets the mask for subnet (IPV6). *)
 val set_ipv6_subnet_mask :
-  ?override_configfile:bool -> int64 * int64 -> unit
+  ?override_configfile:bool -> int -> unit
 
 
 
@@ -715,6 +716,10 @@ val discard_all_services :
 
 module Ext : sig
 
+  (** Exception raised when you try to access a reference
+      belonging to a scope different to the state's scope *)
+  exception Wrong_scope
+
   (** Type used to describe session timeouts *)
   type timeout =
     | TGlobal (** see global setting *)
@@ -774,29 +779,48 @@ module Ext : sig
   (** Discard external states *)
   val discard_state : state : ('a, 'b) state -> unit Lwt.t
 
-  (** Fold all sessions in a groups, or all client processes in a session. *)
+  (** Fold all sessions in a groups, or all client processes in a session.
+      If you do not call the function during
+      a request or during the initialisation phase of the Eliom module,
+      you must provide the extra parameter [?sitedata],
+      that you can get by calling {!Eliom_request_info.get_sitedata}
+      during the initialisation phase of the Eliom module.
+  *)
   val fold_volatile_sub_states :
+    ?sitedata : Eliom_common.sitedata ->
     state : ([< `Session_group | `Session ],
              [< `Data | `Service ] as 'k) state ->
     ('a -> ([< `Session | `Client_process ], 'k) state -> 'a) ->
     'a -> 'a
 
-  (** Iter on all sessions in a groups, or all client processes in a session. *)
+  (** Iter on all sessions in a groups, or all client processes in a session.
+      See {!fold_volatile_sub_states} for explanation about the [?sitedata]
+      parameter.
+  *)
   val iter_volatile_sub_states :
+    ?sitedata : Eliom_common.sitedata ->
     state: ([< `Session_group | `Session ],
             [< `Data | `Service ] as 'k) state ->
     (([< `Session | `Client_process ], 'k) state -> unit) ->
     unit
 
-  (** Fold all sessions in a groups, or all client processes in a session. *)
+  (** Fold all sessions in a groups, or all client processes in a session.
+      See {!fold_volatile_sub_states} for explanation about the [?sitedata]
+      parameter.
+  *)
   val fold_sub_states :
+    ?sitedata : Eliom_common.sitedata ->
     state : ([< `Session_group | `Session ],
              [< `Data | `Pers | `Service ] as 'k) state ->
     ('a -> ([< `Session | `Client_process ], 'k) state -> 'a Lwt.t) ->
     'a -> 'a Lwt.t
 
-  (** Iter on all sessions in a groups, or all client processes in a session. *)
+  (** Iter on all sessions in a groups, or all client processes in a session.
+      See {!fold_volatile_sub_states} for explanation about the [?sitedata]
+      parameter.
+  *)
   val iter_sub_states :
+    ?sitedata : Eliom_common.sitedata ->
     state: ([< `Session_group | `Session ], 'k) state ->
     (([< `Session | `Client_process ], 'k) state -> unit Lwt.t) ->
     unit Lwt.t
