@@ -1,7 +1,6 @@
-
 type ('a, 'b) page = page
 type options = options
 type 'b return = return
-type ('a, 'b) result = result
+type 'a result = result
 
 include "eliom_reg_simpl.mli"
