@@ -214,6 +214,15 @@ module Client_pass(Helpers : Pa_eliom_seed.Helpers) = struct
   let escape_inject context_level orig_expr gen_id =
     let open Pa_eliom_seed in
     let _loc = Ast.loc_of_expr orig_expr in
+    let assert_no_variables typ =
+      let f = function
+        | Ast.TyQuo _ ->
+            Helpers.raise_syntax_error _loc
+              "The type of an injected value must not contain any type variable."
+        | typ -> typ
+      in
+      ignore ((Ast.map_ctyp f)#ctyp typ)
+    in
     match context_level with
       | Escaped_in_client_value_in section ->
           (* {section{ ... {{ ... %x ... }} ... }} *)
@@ -221,6 +230,7 @@ module Client_pass(Helpers : Pa_eliom_seed.Helpers) = struct
             drop_client_value_ctyp
               (get_type Helpers.find_escaped_ident_type gen_id)
           in
+          assert_no_variables typ;
           push_escaped_binding gen_id orig_expr;
           <:expr< ($lid:gen_id$ : $typ$) >>
       | Injected_in _section ->
@@ -229,6 +239,7 @@ module Client_pass(Helpers : Pa_eliom_seed.Helpers) = struct
             drop_client_value_ctyp
               (get_type Helpers.find_injected_ident_type gen_id)
           in
+          assert_no_variables typ;
           <:expr<
             (Eliom_client.Syntax_helpers.get_injection $str:gen_id$ : $typ$)
           >>
