@@ -35,14 +35,18 @@ let do_compile () = !mode <> `Infer
 let do_interface () = !mode = `Interface
 let do_dump = ref false
 
-let create_process ?in_ ?out ?err name args =
-  wait (create_process ?in_ ?out ?err name args)
+let create_process ?in_ ?out ?err ?on_error name args =
+  wait ?on_error (create_process ?in_ ?out ?err name args)
 
 let rec check_or_create_dir name =
   if name <> "/" then
     try ignore(Unix.stat name) with Unix.Unix_error _ ->
       check_or_create_dir (Filename.dirname name);
-      Unix.mkdir name 0o777
+      try Unix.mkdir name 0o777 with
+        (* this append sometime with // compilation *)
+        | Unix.Unix_error (Unix.EEXIST,_,_) -> ()
+        | Unix.Unix_error (_,_,_) ->
+          Printf.eprintf "Unexpected error while creating directory %S" name
 
 let prefix_output_dir name =
   match !build_dir with
@@ -134,14 +138,14 @@ let compile_ocaml ~impl_intf file =
     output_prefix file ^ ext in
   create_process !compiler ( ["-c" ;
                               "-o" ; obj ;
-                              "-pp"; get_pp []]
+                              "-pp"; get_pp [] !ppopt]
                              @ !args
 			     @ get_thread_opt ()
 			     @ get_common_include ()
 			     @ [impl_intf_opt impl_intf; file] )
 
 let output_ocaml_interface file =
-    create_process !compiler ( ["-i"; "-pp"; get_pp []] @ !args
+    create_process !compiler ( ["-i"; "-pp"; get_pp [] !ppopt] @ !args
                                @ get_common_include ()
                                @ [file] )
 
@@ -158,31 +162,40 @@ let compile_obj file =
 
 (* Process eliom and eliomi files *)
 
-let get_ppopts ~impl_intf file =
-  let pa_cmo =
-    match !kind with
-      | `Client -> "pa_eliom_client_client.cmo"
-      | `Server | `ServerOpt -> "pa_eliom_client_server.cmo"
+let get_ppopts ?kind ~impl_intf file =
+  let pkg =
+    match get_kind kind with
+      | `Client -> ["eliom.syntax.client"]
+      | `Server | `ServerOpt -> ["eliom.syntax.server"]
   in
-  pa_cmo :: type_opt impl_intf file @ !ppopt @ [impl_intf_opt impl_intf]
+  pkg, type_opt impl_intf file @ !ppopt @ [impl_intf_opt impl_intf]
 
 let compile_server_type_eliom file =
   let obj = output_prefix ~ty:true file ^ !server_types_file_ext
-  and ppopts = ["pa_eliom_type_filter.cmo"] @ !ppopt @ ["-impl"] in
+  and ppopts = !ppopt @ ["-impl"] in
   if !do_dump then begin
-    let camlp4, ppopt = get_pp_dump ("-printer" :: "o" :: ppopts @ [file]) in
+    let camlp4, ppopt =
+      get_pp_dump ["eliom.syntax.type"] ("-printer" :: "o" :: ppopts @ [file]) in
     create_process camlp4 ppopt;
     exit 0
   end;
   let out = Unix.openfile obj [Unix.O_WRONLY; Unix.O_CREAT; Unix.O_TRUNC] 0o666 in
-  create_process ~out !compiler ( [ "-i" ; "-pp"; get_pp ppopts]
-				  @ !args
-    				  @ get_common_include ()
-				  @ ["-impl"; file] );
+  let on_error _ =
+    Unix.close out;
+    Sys.remove obj
+  in
+  create_process ~out ~on_error
+    !compiler ( [ "-i" ; "-pp"; get_pp ["eliom.syntax.type"] ppopts]
+		@ !args
+    		@ get_common_include ()
+		@ ["-impl"; file] );
   Unix.close out
 
 let output_eliom_interface ~impl_intf file =
-  let ppopts = get_ppopts ~impl_intf file in
+  if !do_dump then begin
+    Printf.eprintf "Dump (-dump) not supported for interface inference (-i).";
+    exit 1
+  end;
   let indent ch =
     try
       while true do
@@ -191,20 +204,20 @@ let output_eliom_interface ~impl_intf file =
       done
     with End_of_file -> ()
   in
+  let args kind =
+    let pkg, ppopts = get_ppopts ~kind ~impl_intf file in
+    let pp = get_pp pkg ppopts in
+    [ "-i" ; "-pp" ; pp; "-intf-suffix"; ".eliomi" ]
+    @ !args
+    @ get_common_include ~kind ()
+    @ [ impl_intf_opt impl_intf; file ]
+  in
   Printf.printf "(* WARNING `eliomc -i' generated this pretty ad-hoc - use with care! *)\n";
   Printf.printf "{server{\n";
-    create_filter !compiler ( [ "-i" ; "-pp" ; get_pp ppopts; "-intf-suffix"; ".eliomi" ]
-			         @ !args
-			         @ get_common_include ~kind:`Server ()
-			         @ [impl_intf_opt impl_intf; file] )
-      indent;
+  create_filter !compiler (args `Server) indent;
   Printf.printf "}}\n";
   Printf.printf "{client{\n";
-    create_filter !compiler ( [ "-i" ; "-pp" ; get_pp ppopts; "-intf-suffix"; ".eliomi" ]
-			         @ !args
-			         @ get_common_include ~kind:`Server ()
-			         @ [impl_intf_opt impl_intf; file] )
-      indent;
+  create_filter !compiler (args `Client) indent;
   Printf.printf "}}\n"
 
 let compile_eliom ~impl_intf file =
@@ -215,15 +228,15 @@ let compile_eliom ~impl_intf file =
     in
     output_prefix file ^ ext
   in
-  let ppopts = get_ppopts ~impl_intf file in
+  let pkg, ppopts = get_ppopts ~impl_intf file in
   if !do_dump then begin
-    let camlp4, ppopt = get_pp_dump ("-printer" :: "o" :: ppopts @ [file]) in
+    let camlp4, ppopt = get_pp_dump pkg ("-printer" :: "o" :: ppopts @ [file]) in
     create_process camlp4 ppopt;
     exit 0
   end;
   create_process !compiler ( [ "-c" ;
                                "-o"  ; obj ;
-                               "-pp" ; get_pp ppopts;
+                               "-pp" ; get_pp pkg ppopts;
                                "-intf-suffix"; ".eliomi" ]
                              @ get_thread_opt ()
 			     @ !args
@@ -233,11 +246,11 @@ let compile_eliom ~impl_intf file =
 
 let process_eliom ~impl_intf file =
   match !mode with
-    | `Infer when impl_intf = `Impl ->
+  | `Infer when impl_intf = `Impl ->
       compile_server_type_eliom file
-    | `Interface ->
+  | `Interface ->
       output_eliom_interface ~impl_intf file
-    | _ ->
+  | _ ->
       compile_eliom ~impl_intf file
 
 let build_server ?(name = "a.out") () =
@@ -263,6 +276,8 @@ let rec process_option () =
   while !i < Array.length Sys.argv do
     match Sys.argv.(!i) with
     | "-help" | "--help" -> usage ()
+    | "-no-autoload" -> autoload_predef := false; incr i
+    | "-type_conv" -> type_conv := true; incr i
     | "-i" -> set_mode `Interface; incr i
     | "-c" -> set_mode `Compile; incr i
     | "-a" -> set_mode `Library; incr i
