@@ -9,6 +9,7 @@ let usage () =
      default_client_dir default_server_dir;
   Printf.eprintf "  -type-dir <dir>\tThe directory for generated type_mli files (default %S)\n"
      default_type_dir;
+  Printf.eprintf "  -eliom-inc <dir>\tAdd <dir> to the list of eliom include directories (prepend eliom build directories)\n";
   Printf.eprintf "  -package <name>\tRefer to package when compiling\n";
   Printf.eprintf "  -ppopt <p>\t\tAppend option <opt> to preprocessor invocation\n";
   Printf.eprintf "  -predicates <p>\tAdd predicate <p> when resolving package properties\n";
@@ -16,21 +17,34 @@ let usage () =
   create_filter !compiler ["-help"] (help_filter 2 "STANDARD OPTIONS:");
   exit 1
 
+(* We use inode for eliom include directories, it's the easier way to
+ * detect if two directories are the same *)
+let inode_of_dir d =
+  (Unix.stat d).Unix.st_ino
+
 (** Context *)
 
 let do_dump = ref false
 
 let mode : [`Normal | `Sort] ref = ref `Normal
 let sort_files = ref []
+let eliom_inc_dirs = ref ["."]
+let eliom_inc_inodes = ref [inode_of_dir "."]
 
 let do_sort () =
   !mode = `Sort
 
+let in_an_eliom_inc_dir s =
+  List.exists
+    (fun d_inode -> inode_of_dir (Filename.dirname s) = d_inode)
+    (!eliom_inc_inodes)
+
 let add_build_dir s =
-  if s = ":" then s else
-  match !build_dir with
-  | "" -> s
-  | d -> d ^ "/" ^ s
+  if s = ":" || not (in_an_eliom_inc_dir s)
+  then s else
+    match !build_dir with
+    | "" -> s
+    | d -> d ^ "/" ^ s
 
 let add_build_dirs line =
   String.concat " " (List.map add_build_dir (split ' ' line ))
@@ -68,54 +82,77 @@ let eliom_synonyms = [ "-ml-synonym"; ".eliom"; "-mli-synonym"; ".eliomi" ]
 
 let compile_intf file =
   create_filter
-    !compiler ( "-pp" :: get_pp !ppopt :: eliom_synonyms @ !args
+    !compiler ( "-pp" :: get_pp [] !ppopt :: eliom_synonyms @ !args
+        @ (map_include !eliom_inc_dirs)
 		@ ["-intf"; file] )
     (on_each_line add_build_dirs)
 
 let compile_impl file =
   create_filter
-    !compiler ( "-pp" :: get_pp !ppopt :: eliom_synonyms @ !args
+    !compiler ( "-pp" :: get_pp [] !ppopt :: eliom_synonyms @ !args
+        @ (map_include !eliom_inc_dirs)
 		@ ["-impl"; file] )
     (on_each_line add_build_dirs)
 
 let server_pp_opt impl_intf =
-  ["pa_eliom_client_server.cmo"; "-notype"] @ !ppopt @ [impl_intf_opt impl_intf]
+  ["-notype"] @ !ppopt @ [impl_intf_opt impl_intf]
 let client_pp_opt impl_intf =
-  ["pa_eliom_client_client.cmo"; "-notype"] @ !ppopt @ [impl_intf_opt impl_intf]
+  ["-notype"] @ !ppopt @ [impl_intf_opt impl_intf]
 let type_pp_opt impl_intf =
-  ["pa_eliom_type_filter.cmo"] @ !ppopt @ [impl_intf_opt impl_intf]
+  !ppopt @ [impl_intf_opt impl_intf]
 
 let compile_server_eliom ~impl_intf file =
   if !do_dump then begin
-    let camlp4, ppopt = get_pp_dump ("-printer" :: "o" :: server_pp_opt impl_intf @ [file]) in
+    let camlp4, ppopt =
+      get_pp_dump
+        ["eliom.syntax.server"]
+        ("-printer" :: "o" :: server_pp_opt impl_intf @ [file]) in
     ignore (create_process camlp4 ppopt);
     exit 0
   end;
   create_filter !compiler
-    ( "-pp" :: get_pp (server_pp_opt impl_intf) :: eliom_synonyms @ !args
+    ( "-pp" ::
+      get_pp
+        ["eliom.syntax.server"]
+        (server_pp_opt impl_intf) :: eliom_synonyms @ !args
+      @ (map_include !eliom_inc_dirs)
       @ [impl_intf_opt impl_intf; file] )
     (on_each_line add_build_dirs)
 
 let compile_type_eliom ~impl_intf file =
   if !do_dump then begin
     (* Won't run because [compile_server_eliom] is first and exits ... *)
-    let camlp4, ppopt = get_pp_dump ("-printer" :: "o" :: type_pp_opt impl_intf @ [file]) in
+    let camlp4, ppopt =
+      get_pp_dump
+        ["eliom.syntax.type"]
+        ("-printer" :: "o" :: type_pp_opt impl_intf @ [file]) in
     ignore (create_process camlp4 ppopt);
     exit 0
   end;
   create_filter !compiler
-    ( "-pp" :: get_pp (type_pp_opt impl_intf) :: eliom_synonyms @ !args
+    ( "-pp" ::
+      get_pp
+        ["eliom.syntax.type"]
+        (type_pp_opt impl_intf) :: eliom_synonyms @ !args
+      @ (map_include !eliom_inc_dirs)
       @ [impl_intf_opt impl_intf; file] )
     (on_each_line server_type_file_dependencies)
 
 let compile_client_eliom ~impl_intf file =
   if !do_dump then begin
-    let camlp4, ppopt = get_pp_dump ("-printer" :: "o" :: client_pp_opt impl_intf @ [file]) in
+    let camlp4, ppopt =
+      get_pp_dump
+        ["eliom.syntax.client"]
+        ("-printer" :: "o" :: client_pp_opt impl_intf @ [file]) in
     ignore (create_process camlp4 ppopt);
     exit 0
   end;
   create_filter !compiler
-    ( "-pp" :: get_pp (client_pp_opt impl_intf) :: eliom_synonyms @ !args
+    ( "-pp" ::
+      get_pp
+        ["eliom.syntax.client"]
+        (client_pp_opt impl_intf) :: eliom_synonyms @ !args
+      @ (map_include !eliom_inc_dirs)
       @ [impl_intf_opt impl_intf; file] )
     (on_each_line add_build_dirs)
 
@@ -134,21 +171,30 @@ let compile_eliom ~impl_intf file =
       Printf.printf "%s.cmx : %s\n" (add_build_dir basename) (get_type_file file) )
 
 let sort () =
-  let ppopt =
+  let pkg, ppopt =
     match !kind with
-      | `Server | `ServerOpt -> server_pp_opt `Impl
-      | `Client -> client_pp_opt `Impl
+      | `Server | `ServerOpt ->
+          ["eliom.syntax.server"], server_pp_opt `Impl
+      | `Client -> ["eliom.syntax.client"], client_pp_opt `Impl
   in
   create_process !compiler
-    ( "-sort" :: "-pp" :: get_pp ppopt :: eliom_synonyms @
+    ( "-sort" :: "-pp" :: get_pp pkg ppopt :: eliom_synonyms @
         List.(concat (map (fun file -> ["-impl"; file]) !sort_files)) )
 
-let rec process_option () =
+let process_option () =
   let i = ref 2 in
   while !i < Array.length Sys.argv do
     match Sys.argv.(!i) with
     | "-verbose" -> verbose := true; incr i
+    | "-no-autoload" -> autoload_predef := false; incr i
+    | "-type_conv" -> type_conv := true; incr i
     | "-sort" -> mode := `Sort; incr i
+    | "-eliom-inc" ->
+      if !i+1 >= Array.length Sys.argv then usage ();
+      let dir = Sys.argv.(!i+1) in
+      eliom_inc_inodes := (inode_of_dir dir) :: !eliom_inc_inodes;
+      eliom_inc_dirs := dir :: !eliom_inc_dirs;
+      i := !i+2
     | "-package" ->
       if !i+1 >= Array.length Sys.argv then usage ();
       package := !package @ split ',' Sys.argv.(!i+1);
@@ -161,6 +207,10 @@ let rec process_option () =
       if !i+1 >= Array.length Sys.argv then usage ();
       pp := Some Sys.argv.(!i+1);
       i := !i+2
+    | "-ppopt" ->
+      if !i+1 >= Array.length Sys.argv then usage ();
+      ppopt := !ppopt @ [Sys.argv.(!i+1)];
+      i := !i+2
     | "-dir" ->
       if !i+1 >= Array.length Sys.argv then usage ();
       build_dir := Sys.argv.(!i+1);
