@@ -27,6 +27,7 @@ let _ = Eliom_registration.Html5.register Eliom_testsuite_base.main
                            ~service:(Eliom_service.static_dir ()) ["ocsigen5.png"]) ()];
 
             Eliom_testsuite_base.testsuite ~name:"Test suite 4" Eliom_testsuite4.tests;
+            Eliom_testsuite_base.testsuite ~name:"Test suite 5" Eliom_testsuite5.tests;
             h3 [pcdata "Eliom examples"];
             h4 [pcdata "Simple pages"];
             p [
@@ -48,9 +49,9 @@ let _ = Eliom_registration.Html5.register Eliom_testsuite_base.main
               a default [code [pcdata "rep/"]] ()
             ];
 
-            h4 [pcdata "Destillery"];
+            h4 [pcdata "Distillery"];
             p [
-              Raw.a ~a:[a_href (Raw.uri_of_string "destillery/basic")] [pcdata "Basic destillery"];
+              Raw.a ~a:[a_href (Raw.uri_of_string "distillery/basic")] [pcdata "Basic distillery"];
             ];
 
             h4 [pcdata "Parameters"];
@@ -298,7 +299,6 @@ let _ = Eliom_registration.Html5.register Eliom_testsuite_base.main
             h3 [pcdata "js_of_ocaml events"];
 
             p [
-              a event_service [code [pcdata "With arrows"]] (); br ();
               a event2_service [code [pcdata "With Lwt"]] ();
               br ();
             ];
@@ -319,6 +319,7 @@ let _ = Eliom_registration.Html5.register Eliom_testsuite_base.main
          a suffixform_su2 [pcdata "Suffix and other service at same URL: a form towards the suffix service"] (); br ();
          a su4 [pcdata "Suffix service with constant part and priority"] ("aa", ((), "bb")); br ();
          a preappliedsuffix [pcdata "Preapplied suffix"] (); br ();
+         a attnonatt_service [pcdata "Attachment of non-attached coservice"] (); br ();
          a constform [pcdata "Form towards suffix service with constants"] (); br ();
          a getact [pcdata "action on GET attached coservice, etc"] 127; br ();
          a noreload [pcdata "action with `NoReload option"] (); br ();
@@ -553,7 +554,7 @@ let _ = Eliom_registration.Html5.register Eliom_testsuite_base.main
                 a tcookies
                   [ code [ pcdata "tcookies" ] ] ();
                 br ();
-                pcdata "A link inside the application that ascks for an action outside the application. Eliom will ask the client side program to so a redirection: ";
+                pcdata "A link inside the application that ask for an action outside the application. Eliom will ask the client side program to do a redirection: ";
                 a actionoutside [ code [ pcdata "actionoutside" ] ] ();
                 br ();
               ]
